from bglobals import *


class top_most:
    def __init__(self):
        pass

    def get_atts(self):
        if not self.second:
            self.kind = 'g'  # this must be set to g in order to work
            # self.kind2 = 'e'
            self.kind2 = 'f'
            bb=8
            self.short = 0
            self.jv = JVReplacer()
            self.alemmas = defaultdict(dict)
            self.co_lemmas1a = pi.open_pickle(f'{fold}co_lemmas1a', 1)
            self.old2new_mods = pi.open_pickle(f'{fold}old2new_mods', 1)
            for k, v in self.co_lemmas1a.items():
                v['macron'] = ""
            self.lat_freq = pi.open_pickle(f'{fold}latin_freq', 1)
            lst = []
            self.ignore_stem = [x[0] for x in lst if len(x) > 1 and x[1] == 0]
            self.false_prefix = [x[0][1:] for x in lst if x[0][0] == '-']
            self.prefix, self.prefix2 = prefixes()
            self.accur = pi.open_pickle(f'{fold}accuracy', 1)
            self.pre3 = vgf.sort_lst_by_len(list(self.prefix.keys()), 1)
            self.pre_dcons = vgf.sort_lst_by_len(list(self.prefix2), 1)
            self.i2j = pi.open_pickle(f'{fold}i2j', 1)
            self.debug = ['peto']
            self.debug = ['aegates']
            self.debug = []
            self.lem2part = defaultdict(dict)
            self.lem2part2 = defaultdict(dict)
        if self.kind == 'g':
            self.form = 'geninf'
        else:
            self.form = 'perf'
        for k, v in self.co_lemmas1a.items():
            v[f'{self.form}m'] = []
        self.diph = ['ae', 'au', 'ei', 'eu', 'oe', 'ui']

        return

    def output(self, kind=1):
        if kind == 6:
            pi.save_pickle(self.stan2alt, f'{fold}stan2alt_spell', 1)
        if kind == 2:
            pi.save_pickle(self.co_lemmas1a, f'{fold}co_lemmas2', 1)
        if kind == 9:
            pi.save_pickle(self.col_dct, f'{fold}col_dct', 1)
        return


class step_three(top_most):
    def __init__(self):
        top_most.__init__(self)

    def quick_fix(self):
        if self.kind2 == 'e':
            return
        for k, v in self.co_lemmas1a.items():
            if k == 'crebresco':
                bb = 8

            if v['geninfm']:
                lst = [x for x in v['geninfm'] if not reg(r'[\)\(\-#]', x)]
                v['geninfm'] = lst
                self.co_lemmas1a[k] = v
            if v['perfm']:
                lst = [x for x in v['perfm'] if not reg(r'[\)\(\-#]', x)]
                v['perfm'] = lst
                self.co_lemmas1a[k] = v

    def fdiph_fu(self, med=0):
        if med:
            self.co_lemmas1a = pi.open_pickle(f'{fold}flemmas', 1)
            self.quick_fix()
            self.i2j = pi.open_pickle(f'{fold}i2j', 1)
        if not self.debug and not self.kind2 in ['e', 'f']:
            self.co_lemmas1a['nereides']['geninfm'] = ['nēried']
            self.co_lemmas1a['adrastis']['geninfm'] = ['ādrastid']
            self.co_lemmas1a['aeetias']['geninfm'] = ['aeētiad']
            self.co_lemmas1a['sanguinans']['geninfm'] = ['sanguinānt']

        self.counts = 0
        self.tjv = set()
        self.has_fd = {}
        self.errors_fd = {}
        if self.kind2 == 'e':
            lemmas = self.alemmas
        else:
            lemmas = self.co_lemmas1a

        auth_list = ['LS', 'GG', 'GJ', 'Ge', 'Lw', 'YO', 'PO']
        b =0
        for k, obj in lemmas.items():
            b += 1
            vgf.print_intervals(b,1000, None, len(lemmas))
            if self.kind2 == 'e':
                authors = auth_list
            else:
                authors = [0]

            for auth in authors:
                if auth:
                    v = obj.get(auth)
                else:
                    v = obj

                if v:
                    if k == 'ambiguitas':
                        bb = 8
                    self.lem_name = re.sub(r'[0-9]', '', v['lemma'])
                    if not 'geninfm' in v and self.kind2 == 'e':
                        pass
                    else:
                        v['geninfm'] = list(set(v['geninfm']))
                        v['perfm'] = list(set(v['perfm']))

                        if self.kind2 == 'e':
                            syl = v['macronjv']
                        else:
                            if not v['syl']:
                                syl = v['spell']
                            else:
                                syl = v['syl']
                        if not self.kind2 == 'e':
                            syl = norm_str_wmac(syl)
                            self.cat = 'macron'
                            self.step2(syl, v, k)
                            self.cat = 'geninfm'
                            syl = 0
                            if v[self.cat]:
                                bool1 = self.join_str(v)
                                if bool1:
                                    self.step2(syl, v, k)
                            self.cat = 'perfm'
                            if v[self.cat]:
                                bool1 = self.join_str(v)
                                if bool1:
                                    self.step2(syl, v, k)

        return

    def check_jv(self):
        '''
        the following are still missing
        ('accersio', 'accersiu')
        ('mauiael', 'mauiael')
        ('percompleo', 'percompleu')
        ('ioculatoria', 'ioculatoria')
        ('euilmerodach', 'euilmerodach')
        ('heuilath', 'heuilath')
        ('heuila', 'heuila')
        ('indignatiuum', 'indignatiuum')
        ('bethauen', 'bethauen')
        ('negatiua', 'negatiua')
        ('sepharuaim', 'sepharuaim')
        ('heuaeus', 'heuaeus')
        ('iuncina', 'iuncina')
        ('uolupe', 'uolup')
        ('c', 'inu')
        '''
        if self.kind2 == 'e':
            return
        miss = set(self.i2j.keys()) - self.tjv
        miss2 = self.tjv - set(self.i2j.keys())
        return

    def join_str(self, v):
        if v == 'z': return 0
        str1 = v[f'o{self.cat[:-1]}']
        str1_wom = unidecode(str1).lower()
        str1_wom = jv.replace(str1_wom)
        woml = str1_wom.split(',')
        dct = {}
        wo2mac = {}
        if type(v[self.cat]) == str:
            v[self.cat] = v[self.cat].split(',')

        for x in v[self.cat]:
            xwom = unidecode(x)
            wo2mac[xwom] = x
            found = 0
            for e, u in en(woml):
                if u == xwom:
                    dct[xwom] = e
                    found = 1
                    break
            if not found:
                self.errors_fd[v['lemma']] = v
        dct = sort_dct_val(dct)
        lst = []
        for x, y in dct.items():
            lst.append(wo2mac[x])
        str2 = ",".join(lst)
        str2_wom = unidecode(str2)
        if str2_wom != str1_wom:
            self.errors_fd[v['lemma']] = v
        v[self.cat] = str2

        return 1

    def step2(self, syl, v, k):
        if not syl:
            syl = v[f'o{self.cat[:-1]}'].lower()
            # if not syl:
            #     syl = v[f'{self.cat[:-1]}'].lower()

        syl2, fd, uw = remove_hats_diph(syl, syl, 1)
        if uw:
            self.add_uw(uw, v, syl)
        if fd:
            v[self.cat] = self.restore(fd, v, syl2)

        if fd or uw:
            self.has_fd[k] = v
        self.add_jv(v[self.cat], v)

    def add_uw(self, uw, v, syl):
        mac = v[self.cat]
        for i in uw:
            mac = replace_at_i(i, mac, chr(7909))
        v[self.cat] = mac

    def restore(self, fd, v, syl2):
        mac = v[self.cat]
        wo_mac = unidecode(syl2)
        brev = 'ăĕĭŏŭў'
        mac2 = ""
        found = 0
        for i in fd:
            lets = wo_mac[i:i + 2]
            letsm = syl2[i:i + 2]
            if lets in self.diph:
                if any(x in letsm for x in brev):
                    found = 1
                    mac2 = mac[:i] + syl2[i:i + 2]
                    if len(mac) > i + 2:
                        mac2 += mac[i + 2:]
                    else:
                        bb = 8
        if found:
            return mac2
        else:
            return mac

    def add_jv(self, mac3, v):
        if ',' in mac3:
            lst = mac3.split(',')
            lst1 = []
            for x in lst:
                y = self.add_jv2(x)
                lst1.append(y)
            y = ','.join(lst1)
        else:
            y = self.add_jv2(mac3)
        v[f'{self.cat}jv'] = y
        return

    def add_jv2(self, x):
        if x == 'adseruo':
            bb = 8
        macp = unidecode(x)
        lst = self.i2j.get((self.lem_name, macp), [[], []])
        if lst[0] or lst[1]:
            self.tjv.add((self.lem_name, macp))
            self.counts += 1

        for j in lst[0]:
            x = replace_at_i(j, x, 'j')
        for v in lst[1]:
            x = replace_at_i(v, x, 'v')
        return x


class step_two(step_three):
    def __init__(self):
        step_three.__init__(self)

    def begin_cl2(self):
        self.get_values()
        self.strincomm()  ## need an interval here
        self.get_lem_by_auth()
        self.still_missing()
        if self.kind2 == 'e':
            self.each_author()

    def get_values(self):  # 13062
        self.lem2part3 = defaultdict(dict)
        self.lem2part4 = defaultdict(dict)
        if self.kind == 'p':
            form = 'perf'
        else:
            form = 'geninf'

        c = 0
        self.no_geninf = defaultdict(dict)
        mult = defaultdict(dict)
        self.geninf2lem = defaultdict(set)

        for k, v in self.lem2part.items():  # todo put an interval here
            # p (k)
            lemma = self.co_lemmas1a[k]
            geninf = lemma[form]
            if geninf:
                c += 1
                geninf = norm_str_jv(geninf)
                geninf2 = tuple(vgf.strip_n_split(geninf, ","))
                for x, z in v.items():
                    if k == 'coerceo':
                        bb = 8

                    found = 1
                    got = 0
                    y = norm_str_jv(z)
                    geninf3 = 0
                    if all(t not in y for t in geninf2):
                        geninf3 = tuple(self.word2prefix.get(u, ("", u)) for u in geninf2)
                        found = 2
                        if all(t[1] not in y for t in geninf2):
                            self.no_geninf[k][x] = [geninf, y]
                            self.lem2part3[k][x] = [geninf, y, 0]
                            got = 1
                            found = 0

                    if not geninf3:
                        geninf3 = tuple(("", q) for q in geninf2)

                    if found:
                        if found == 2 and len(geninf2) > 1:
                            bb = 8
                        for gf in geninf3:
                            pre = gf[0]
                            gf = gf[1]
                            self.geninf2lem[(pre + gf), x].add(k)
                            if gf in y:
                                if len(geninf2) > 1:
                                    bb = 8
                                b = y.count(gf)
                                z = self.jv.replace(z)
                                lst = [gf, z]
                                for i in range(b):
                                    idx = y.index(gf)
                                    s = z[idx:idx + len(gf)]
                                    y = y[idx + len(gf):]
                                    z = z[idx + len(gf):]
                                    if pre:
                                        if gf[-1] == gf[0] and gf[:-1] in self.prefix2:
                                            pre_mac = pre
                                        else:
                                            pre_mac = self.prefix.get(pre)
                                        s = f'{pre}{s}'
                                    s = s.lower()
                                    lst.append(s)
                                got = 1
                                self.lem2part3[k].setdefault(x, []).append(lst)
                                if len(lst) > 3:
                                    mult[k][x] = lst
                    if not got:
                        self.lem2part3[k][x] = [geninf, y, 0]

        if c != len(self.lem2part3):
            p('missing in lem2part3')

        return

    def each_author(self):
        lst1 = ['LS', 'GG', 'GJ', 'Ge', 'Lw', 'YO', 'PO']
        b = 0
        for k, v in self.col_dct.items():
            already = 0
            for z in v:
                auth = z[0]
                if auth in lst1 and len(z) > 2:
                    if not self.second:
                        lem = new_lem2(k)
                        lem['lemma'] = z[2]
                        obj2 = self.co_lemmas1a[k]
                        lem['lexicon'] = obj2['lexicon']
                        lem['quantity'] = ""
                        lem['model'] = obj2['model']
                        lem['model'] = self.old2new_mods.get(lem['model'], lem['model'])
                        lem[self.form] = ""

                    else:
                        obj = self.alemmas.get(k)
                        if obj:
                            lem = obj[auth]
                    em = self.emiss.get(k)
                    if em:
                        if auth in em:
                            lem[f'{self.form}'] = em[auth]
                            if not already:
                                b += 1
                            already = 1
                    else:

                        obj = self.lem2part3.get(k)
                        if obj:
                            itm = obj.get(auth)
                            if itm:
                                if type(itm[0]) == list and itm[0][2]:
                                    lst4 = [y[2] for y in itm]
                                    d = ','.join(lst4)
                                    # if d != lem['lemma']:
                                    lem[f'{self.form}'] = d

                    if not self.second:
                        self.alemmas[k][auth] = lem

        return

    def adhoc_comma(self):
        lst = to.from_txt2lst_tab_delim(f'{fold}stem_comma')
        lst1 = to.from_txt2lst_tab_delim(f'{fold}stem_comma_got')
        dct4 = {}
        for x in lst:
            dct4[x[0], x[2]] = f'{x[4]},{x[5]}'
        for x in lst1:
            dct4[x[0], x[2]] = x[4]
        for x, y in dct4.items():
            obj = self.alemmas[x[0]]
            for k, v in obj.items():
                stem = x[1]
                s = y
                v[stem] = s

    def check_alemma(self):
        if self.second:
            '''
            tharsis: thars,tharsid
            iaiunitas: 'iaiunitat,ieiunitat'
            saeclaris: 'saeclar,saecular'
            '''

            dct = defaultdict(dict)
            for k, v in self.alemmas.items():
                lem = self.co_lemmas1a.get(k)
                if lem:
                    for l in ['geninf', 'perf']:
                        s = lem.get('o' + l)
                        if s:
                            gnum = s.count(',')
                            num = 0
                            ts = ''
                            for x, y in v.items():
                                t = y.get(l)
                                if t:
                                    if not ts:
                                        ts = t
                                    num1 = t.count(',')
                                    if num1 > num:
                                        num = num1
                                        ts = t
                            if num != gnum:
                                dct[k][l] = [lem[l], ts]
            if len(dct) > 1:
                p(f'{len(dct)} still have wrong commas')
                self.fix_stem_alt(dct)
        return

    def fix_stem_alt(self, dct):
        by_hand2 = []
        b = 0
        got = []
        lst7 = []
        for k, v in dct.items():
            for stem, y in v.items():
                orig = y[0]
                target = y[1]
                mlem = self.get_mac(k)
                if not target:
                    lst7.append([k, mlem, stem, orig])

                else:
                    targetu = unidecode(target)
                    ost = set(orig.split(','))
                    tst = targetu.split(',')
                    tst = [remove_hats_diph(f, f) for f in tst]
                    tst_or = target.split(',')
                    rem = ost - set(tst)
                    have = ost & set(tst)
                    tar_lst = target.split(',')
                    have_ma = []
                    for s in tar_lst:
                        su = unidecode(s)
                        if su in have:
                            have_ma.append(s)

                    lst = []
                    by_hand = []
                    for w in rem:
                        found = 0
                        vow1 = elim_cons(w)
                        for z in tst:
                            vow2 = elim_cons(z)
                            if vow1 == vow2:
                                b += 1
                                for s, com in zip(tst, tst_or):
                                    if s == z:
                                        break

                                nword = match_vowels(com, w)
                                lst.append(nword)
                                found = 1
                                break
                        if not found:
                            by_hand.append(w)
                    if by_hand:
                        by_hand = [k, mlem, stem, orig, target] + by_hand
                        by_hand2.append(by_hand)
                    else:
                        o = ','.join(have_ma + lst)
                        lst4 = [k, mlem, stem, orig, o]
                        got.append(lst4)
        return

    def get_mac(self, k):
        for auth in authors3:
            try:
                return self.alemmas[k][auth]['lemma']
            except:
                pass

    def strincomm(self, debug=0):
        if debug:
            self.lem2part3 = pi.open_pickle(f'{fold}lem2part3', 1)
            self.disg2 = pi.open_pickle(f'{fold}disg2', 1)
        dct = defaultdict(dict)
        hi2mi = {}
        cends = ['e', 'is', 'ium', 'um', 'ae', 'us', 'eos', 'idis', 'a',
                 'euntis', 'm.', 'f.', 'n.', 'ydis', 'ycis', 'i', 'es', 'yos', 'indecl.', 'ii', 'ius']
        self.mixed = defaultdict(dict)
        most = defaultdict(int)
        miss = defaultdict(dict)
        p (f'now mapping participles to lemmas')
        self.diff_stem = defaultdict(dict)
        b = 0
        for k, v in self.lem2part3.items():  # todo another interval here
            b += 1
            if self.kind == 'g':
                vgf.print_intervals(b,200,None,len(self.lem2part3))
            for x, y in v.items():
                if y[-1] == 0 and y[1]:
                    lemma = self.co_lemmas1a.get(k)
                    lemma = lemma['lemma']
                    lemma_mac = self.disg2[lemma][x]
                    if k == 'coerceo':
                        bb = 8

                    u = y[1].strip()
                    o = u
                    gns = y[0]
                    lst5 = []
                    gns_lst = gns.split(',')
                    mginf = []
                    got_ginf = []
                    lst = u.split()
                    fword = lst[0]
                    fword = fword.replace(',', '')
                    if any(cend == fword for cend in cends) or \
                            u in ['ire', 'are', 'ere']:
                        if all(not lemma.startswith(g) for g in gns_lst):
                            bad, good = self.almost_inside(gns_lst, lemma, lemma_mac)
                            if good:
                                y[-1] = good
                                if bad:
                                    self.mixed[k][x] = [y, bad]

                    elif '&mdash' in u:
                        pass
                    else:
                        most[o] += 1

                        best_fit = defaultdict(dict)
                        best_left = defaultdict(dict)
                        for g in gns_lst:
                            lst1 = list(g)
                            for e, s in en(lst):
                                t = norm_str_jv(s)
                                t = re.sub(r'[^a-z]', '', t)
                                lst2 = list(t)
                                n = 1
                                lidx = 0
                                mx = 0
                                while n < len(lst2):
                                    right = lst2[:n]
                                    left = lst1[-n:]
                                    if right == left:
                                        mx = n
                                        lidx = len(lst1) - n
                                    n += 1
                                if mx:
                                    best_fit[g][s] = mx
                                    best_left[g][s] = lidx

                        if best_fit:
                            for g, dc in best_fit.items():  ## use self.disg2, get str3 from there according to author then cut str
                                dc = sort_dct_val_rev(dc)
                                str2 = vgf.dct_idx(dc)
                                lidx = best_left[g][str2]
                                str2 = norm_str_wmac(str2)
                                str2 = remove_hats(str2)
                                mx = vgf.dct_idx(dc, 0, 'v')
                                str2 = str2[:mx]
                                str3 = g[:lidx]
                                lemma2 = lemma[:lidx]
                                lemma_mac, cidx = self.adj_cobr(lemma_mac)
                                nstr = lemma_mac[:lidx] + str2
                                nstr = self.rest_cobr(nstr, cidx)
                                if lemma2 != str3:
                                    self.diff_stem[k][x] = [g, o, nstr]
                                else:
                                    lst5.append(nstr)
                                    got_ginf.append(g)
                        else:
                            mginf.append(g)
                        if lst5:
                            if mginf:
                                hi2mi[tuple(got_ginf)] = mginf
                            lst6 = []
                            for t in lst5:
                                t = t.lower()
                                lst6.append([g, o, t])
                            self.lem2part3[k][x] = lst6
                            dct[k][x] = lst6
                        else:
                            self.lem2part3[k][x] = y
                            miss[k][x] = y

        return

    def adj_cobr(self, x):
        if cobr in x:
            idx = x.index(cobr)
            x = x.replace(cobr, "")
            return x, idx
        return x, 0

    def rest_cobr(self, x, cidx):
        if cidx:
            return add_at_i(cidx, x, cobr)
        return x

    def stem_head(self, k, v):
        olem = self.co_lemmas1a[k]
        s = olem[f'o{self.form}'].lower()
        s = s.replace(cobr, '')
        alt = self.stan2alt.get(k)
        syl = olem['syl']
        syl = remove_hats_diph(syl, syl)
        kvowels = re.sub(r'[^aeiouyāēīōūȳăĕĭŏŭў]', '', syl)
        lst = s.split(',')
        lst = [remove_hats_diph(x, x) for x in lst]
        for auth, alst in v.items():
            tword = self.lem_by_auth[k][auth]
            lst1 = []
            for word in lst:
                vowels = re.sub(r'[^aeiouyāēīōūȳăĕĭŏŭў]', '', word)
                if kvowels.startswith(vowels):
                    nstem = match_vowels(tword, word)
                    lst1.append(nstem)
                else:
                    self.missing[(k, syl)].add(word)
            alst[0][2] = ",".join(lst1)

        return

    def get_lem_by_auth(self):
        dct = defaultdict(dict)
        for k, v in self.col_dct.items():
            for z in v:
                if z[0] in authors:
                    if len(z) > 2:
                        dct[k][z[0]] = z[2]
        self.lem_by_auth = dct

    def still_missing(self):
        '''
        for this class we have already tested that all vowels
        are the same in the geninf and the lemma
                #aborigines, abcido
        '''
        '''
        currently there are no lsts in lem2part3 in the dictionary
        which are longer than 1
        '''

        for k, v in self.lem2part3.items():
            found = 0
            for x, y in v.items():
                if type(y[0]) == str:
                    v[x] = [y]
                    found = 1
            if found:
                self.lem2part3[k] = v

        err_corr = to.from_txt2lst_tab_delim(f'{fold}{self.form}_errors_corr')
        err_corr2 = to.from_txt2lst_tab_delim(f'{fold}{self.form}_errors_corr2')
        err_corr = {x[0]: x[2] for x in err_corr}
        err_corr2 = {x[0]: x[2] for x in err_corr2}
        err_corr = merge_2dicts(err_corr, err_corr2)
        self.missing = defaultdict(set)
        self.mac_roots = {}
        self.emiss = defaultdict(dict)
        setattr(self, f'{self.form}_errors', {})
        dct_err = getattr(self, f'{self.form}_errors')
        wrong_roots = []
        for k, v in self.lem2part3.items():
            if k in err_corr:
                self.add2v(k, v, err_corr)
            else:
                if all(z[2] == 0 for x, y in v.items() for z in y):
                    if k == 'coerceo':
                        bb = 8
                    self.stem_head(k, v)

        p(f'still missing {len(self.missing)}')
        to.from_lst2txt_tab_delim(wrong_roots, f'{fold}{self.form}_errors')
        return

    def add2v(self, k, v, err_corr):
        s = err_corr[k]
        for x, y in v.items():
            y[0][2] = s

    def temp24(self):
        lst = []
        dct2 = {}
        for x, y in self.missing.items():
            s = list(y)[0]
            lst.append([x[0], x[1], s])
            dct2[x[0]] = self.lem_by_auth[x[0]]
        to.from_lst2txt_tab_delim(lst, f'{fold}{self.form}_errors2')

    def almost_inside(self, gns, lemma, lemma_mac):
        lemma_mac_nc, cidx = self.adj_cobr(lemma_mac)
        if cidx:
            bb = 8
        lst = []
        found = 0
        fine = []
        for g in gns:
            b = 0
            for x, y in zip(g, lemma):
                if x != y:
                    rem = g[b:]
                    if reg(r'[aeiouy]', rem):
                        lst.append([g, rem])
                    else:
                        str1 = lemma_mac_nc[:b] + g[b:]
                        str1 = self.rest_cobr(str1, cidx)
                        fine.append(str1)
                        found = 1
                    break
                b += 1
        if cidx and not found:
            bb = 8

        if lst and fine:
            bb = 8

        return lst, ','.join(fine)


class step_one(step_two):

    def __init__(self):
        '''
        kind indicates whether we are parsing geninf (genitive/infinitive) or perf, g for
        geninf, p for perf (perfect)

        '''

        step_two.__init__(self)

    def fix_qerrors(self):
        dct = {
            'a_': 'ā',
            'e_': 'ē',
            'i_': 'ī',
            'o_': 'ō',
            'u_': 'ū',
            'y_': 'ӯ',
        }
        self.get_atts()
        self.step1()
        self.mlst = to.from_txt2lst_tab_delim(f'{fold}quantity_errors')
        still_miss = []
        for l in self.mlst:
            obj = self.col_dct.get(l[2])
            if not obj:
                still_miss.append(l)
            else:
                done = 0
                for auth in authors3:
                    if done:
                        break
                    for e, r in en(obj):
                        if r[0] == auth:
                            if len(r) > 2:
                                nword = r[2]
                                nword = nword.lower()

                                if e == 6:
                                    nword = nword.split('=')[0]
                                elif e == 7:
                                    str1 = nword
                                    for s, t in dct.items():
                                        str1 = str1.replace(s, t)
                                    if ',' in str1:
                                        lst4 = str1.split(',')
                                        str1 = lst4[0]
                                    if ':' in str1:
                                        lst4 = str1.split(':')
                                        str1 = lst4[0].strip()
                                    assert ' ' not in str1
                                    nword = str1

                                nword = remove_hats_diph(nword, nword)
                                done = 1
                                l[2] = nword
                                break

        self.still_miss = still_miss
        to.from_lst2txt_tab_delim(self.mlst, f'{fold}quantity_errors_corr')
        to.from_lst2txt_tab_delim(still_miss, f'{fold}quantity_errors2')

    def normalize_perf(self):
        # todo there are a lot of false diphthongs in the geninf
        # and perf stems because collatinus treats āe to mean that
        # the diphthong ae is long

        for k, v in self.co_lemmas1a.items():
            if v['geninf']:
                g = v['geninf']
                if g == '-':
                    g = ""
                v['ogeninf'] = g.lower()
                v['geninf'] = norm_str_jv(g)
            if v['perf']:
                g = v['perf']
                if g == '-':
                    g = ""
                v['operf'] = g.lower()
                v['perf'] = norm_str_jv(g)
        return

    def elim_prefix(self, word, targets, fake=0):
        lst = []
        for pre in self.pre3:
            if len(word) > len(pre) and word.startswith(pre):

                n = word[len(pre):]
                if n in targets:
                    return n, self.prefix[pre], pre, 0
        for pre in self.pre_dcons:
            if len(word) > len(pre) + 1 and word.startswith(pre) \
                    and word[len(pre)] == word[len(pre) + 1]:
                n = word[len(pre) + 1:]
                pre_dc = word[:len(pre) + 1]
                if n in targets:
                    return n, pre_dc, pre_dc, 0
                elif fake:
                    lst.append([n, pre_dc, pre_dc, 1])

        if fake:
            if lst:
                return lst[0]

            for pre in self.pre3:
                if len(word) > len(pre) and word.startswith(pre):
                    n = word[len(pre):]
                    return n, self.prefix[pre], pre, 0

        return 0, 0, 0, 0

    def step1(self):
        '''
        here we convert the lem_coll5 sheet into the col_dct method

        FG does not have vowel quantity
        YO has an = inbetween sometimes, just use first for now
        PO has _ instead of a macron
        WW can also be ignored
        Lw has syllables
        skip 4, 8 ww
        combined breve chr 774

        impes does not go into flemmas
        '''
        lems = []
        done = 0
        lst = to.from_txt2lst(f'{fold}lem_coll5')
        lst1 = to.from_txt2lst(f'{fold}lem_coll_extra')
        lst += lst1
        lst = [x.split('|') for x in lst]
        dct = defaultdict(list)
        lst1 = ['LS', 'GG', 'GJ', 'Ge', 'FG', 'Lw', 'YO', 'PO', 'WW']
        b = 0
        for e in range(0, len(lst), 10):
            for z in lst[e]:
                if 'impes=' in z:
                    # p('hey')
                    bb = 8
                if hl(z):
                    break

            if done:
                break
            for i in range(0, 9):
                str2 = lst1[i]
                str1 = lst[i + e + 1][0]
                if i == 6 and str2.startswith('Y'):
                    lst[i + e + 1][0] = 'YO'
                elif str1 != str2:
                    p(f'error {e}')
                    done = 1
                    break
                dct[z].append(lst[i + e + 1])
            b += 1
            if self.short:
                if b > 10_000:
                    break
        self.col_dct = dct

        if 'c' in self.col_dct:
            del self.col_dct['c']

        if self.debug:
            self.col_dct = {k: v for k, v in self.col_dct.items() if k in self.debug}
        return

    def fix_lw(self):
        jv = JVReplacer()
        bad = ['(', ',', ' or ', ' ']
        for k, v in self.col_dct.items():
            str2 = unidecode(k.lower())

            if str2[-1].isdigit():
                str2 = str2[:-1]

            if len(v[5]) > 2:
                str8 = v[5][2]
                str8 = str8.lower()
                str8 = jv.replace(str8)
                if str8[0] == '(':
                    str8 = str8[1:]
                if str8[-1] == ')':
                    str8 = str8[:-1]

                str1 = str8
                found = 0
                if ' - ' in str1:
                    idx = str1.index('-')
                    if "(" in str1:
                        idx2 = str1.index('(')
                        if idx2 > idx:
                            str1 = str1[:idx2].strip()
                        else:
                            assert 0

                    str3 = str1.replace(' - ', '')
                    str4 = unidecode(str3)
                    if str2 == str4:
                        str1 = str3
                        found = 1
                if not found:
                    dct7 = {}
                    for bd in bad:
                        if bd in str1:
                            dct7[bd] = str1.index(bd)
                    if dct7:
                        dct7 = sort_dct_val(dct7)
                        bd = vgf.dct_idx(dct7, 0, 'v')
                        str1 = str1[:bd]
                str1 = str1.replace('-', '')
                str1u = unidecode(str1)
                if str1u != str2:
                    p(f'{str1}    {str8}  {k}')
                str1 = remove_hats(str1)
                assert ' ' not in str1
                v[5][2] = str1
                try:
                    if v[5][3] and "," in v[5][3] and self.research_ostem(k):
                        self.lem2part[k]['Lw'] = v[5][3]
                except:
                    pass

        return

    def alternate_spell(self):
        ndct = {}
        self.stan2alt = {}
        for k, v in self.col_dct.items():
            if '=' in k:
                lst = k.split('=')
                self.stan2alt[lst[0]] = lst[1]
                ndct[lst[0]] = v
            else:
                ndct[k] = v
        self.col_dct = ndct

    def jv_rep(self):

        jv = JVReplacer()
        dct = {}
        b = 0
        for k, v in self.col_dct.items():
            k1 = jv.replace(k)
            if k1 in self.co_lemmas1a:
                dct[k1] = v

        self.col_dct = dct
        self.co_lemmas1a = {k: v for k, v in self.co_lemmas1a.items() if k in self.col_dct}
        return

    def fix_po(self):
        self.po_altsp = defaultdict(list)
        dct = {
            'a_': 'ā',
            'e_': 'ē',
            'i_': 'ī',
            'o_': 'ō',
            'u_': 'ū',
            'y_': 'ӯ',
        }
        idx = 2
        for k, v in self.col_dct.items():
            if len(v[7]) > 2:
                str1 = v[7][idx]
                str1 = str1.lower()
                str1 = self.jv.replace(str1)
                for s, t in dct.items():
                    str1 = str1.replace(s, t)

                if ',' in str1:
                    # if idx
                    str2 = str1[str1.index(',') + 1:]
                    if self.research_ostem(k):
                        self.lem2part[k]['PO'] = str2

                    lst4 = str1.split(',')
                    str1 = lst4[0]

                if ':' in str1:
                    lst4 = str1.split(':')
                    str1 = lst4[0].strip()

                assert ' ' not in str1

                v[7][2] = str1

        for k, v in self.col_dct.items():
            if len(v[7]) > 2:
                kd = self.del_num(k)
                z = v[7][2]
                oz = z
                if k == 'asylum':
                    bb = 8

                zu = unidecode(z)

                if zu != kd:
                    # p(f'{zu} {k} {oz}')
                    v[7][2] = ''
                else:
                    z = remove_hats(z)
                    v[7][2] = z

        return

    def del_num(self, x):
        if x[-1].isdigit():
            return x[:-1]
        return x

    def research_ostem(self, k):
        if self.co_lemmas1a[k].get('ogeninf') or self.co_lemmas1a[k].get('operf'):
            return 1

    # def redundant_stems(self):
    #     erased = 0
    #     '''
    #     i think this messed up the old decliner but not the new
    #     one so might be redundant
    #     '''
    #
    #     for k, v in self.lemmas.items():
    #         if v.get('operf') and v['operf'] == v['syl']:
    #             v['operf'] = ""
    #             v['perf'] = ""
    #             erased += 1
    #         if v.get('ogeninf') and v['ogeninf'] == v['syl']:
    #             v['ogeninf'] = ""
    #             v['geninf'] = ""
    #             erased += 1
    #     return

    def fix_ge(self):
        for k, v in self.col_dct.items():
            if len(v[3]) > 2:
                str1 = v[3][2]
                if str1:
                    kd = self.del_num(k)
                    str1 = self.jv.replace(str1)
                    str1 = str1.lower()
                    str1 = str1.replace('-', '')
                    str1 = str1.replace('*', '')
                    str1 = remove_hats(str1)
                    str1u = unidecode(str1)

                    if str1u != kd:
                        if not public:
                            p(f'ill-formed: {kd} {str1}')
                    assert ' ' not in str1
                    v[3][2] = str1
                try:
                    if v[3][3] and self.research_ostem(k):
                        self.lem2part[k]['Ge'] = v[3][3]
                except:
                    pass

        return

    def build_prefix_dct(self):
        self.word2prefix = {}
        self.all_geninf = set()
        self.fake_prefixes = set()
        excep = ['aetias', 'aeetis']
        if self.kind == 'p':
            form = 'perf'
        else:
            form = 'geninf'
        for k, v in self.lem2part.items():
            lemma = self.co_lemmas1a.get(k)
            if lemma[form] and lemma['model'] not in greek_models() \
                    and k not in excep:
                s = lemma[form]
                t = self.jv.replace(s)
                g = set(t.split(','))
                g = set(h.lower() for h in g)
                self.all_geninf |= g

        for w in self.all_geninf:
            y, z, pre, fake = self.elim_prefix(w, self.all_geninf, 1)
            if y:
                if fake and y not in self.ignore_stem:
                    self.fake_prefixes.add(y)
                self.word2prefix[w] = (pre, y)
        assert 'admeto' not in self.word2prefix
        return

    def jv_others(self):
        p(f'putting the macrons into a standard form')
        b = 0
        for k, v in self.col_dct.items():
            b += 1
            vgf.print_intervals(b,1000,None,len(self.col_dct))
            for i in range(0, 3):
                if len(v[i]) > 2:
                    str1 = self.jv.replace(v[i][2])
                    str1 = str1.lower()
                    str1 = remove_hats(str1)
                    if ' ' in str1:
                        # p (f'SPACE {str1}')
                        str1 = str1[:str1.index(' ')]

                    v[i][2] = str1
                try:
                    if self.research_ostem(k):
                        self.lem2part[k][v[i][0]] = v[i][3]
                except:
                    pass
        return

    def fix_yo(self):
        for k, v in self.col_dct.items():
            if len(v[6]) > 2:
                ku = self.del_num(k)
                str1 = v[6][2]
                ostr = str1
                str1 = self.jv.replace(str1)
                str1 = str1.split('=')[0]
                str1 = str1.lower()
                str1 = self.del_num(str1)
                str1u = unidecode(str1)
                if str1u != ku:
                    p(f'error in yo {k} {ostr} {str1}')
                no_hats = str1
                str1 = remove_hats(str1)
                assert ' ' not in str1
                v[6][2] = str1
                try:
                    if not v[6][4] and v[6][6]:
                        self.lem2part[k]['YO'] = self.jv.replace(v[6][6].lower())
                    elif v[6][4]:
                        self.lem2part[k]['YO'] = self.jv.replace(v[6][4].lower())
                    if v[6][5]:
                        self.lem2part2[k]['YO'] = self.jv.replace(v[6][5].lower())
                except:
                    pass

        return

    def lem2vquant(self, dct4, dct5, cap):
        ge = dct5.get('PO')
        if ge:
            return ge
            # if ge[0].isupper():
            #     return ge.capitalize()
            # else:
            #     return ge

        if 'YO' in dct4 and 'GJ' in dct4 and len(dct4) == 2:
            return dct5['YO']

        if len(dct5) == 1:
            # if cap:
            #     return vgf.dct_idx(dct5, 0, 'v').capitalize()
            # else:
            return vgf.dct_idx(dct5, 0, 'v')
        else:
            dct7 = defaultdict(int)
            for k, v in dct4.items():
                if k not in ['YO', 'GJ']:
                    dct7[v] += 1
            dct7 = sort_dct_val_rev(dct7)
            if len(dct7) == 1:
                # if cap:
                #     return vgf.dct_idx(dct5, 0, 'v').capitalize()
                # else:
                return vgf.dct_idx(dct5, 0, 'v')
            else:
                num1 = vgf.dct_idx(dct7, 0, 'v')
                num2 = vgf.dct_idx(dct7, 1, 'v')
                word1 = vgf.dct_idx(dct7, 0, 'k')
                if num1 > num2:
                    word = vgf.dct_idx(dct7)
                    for k, v in dct5.items():
                        if word in v:
                            return v
                    # if cap:
                    #     return word.capitalize()
                    # else:
                    assert 0

                else:
                    dct8 = {}
                    lst = []
                    for x, y in dct7.items():
                        if y == num1:
                            lst.append(x)
                    for x, y in dct4.items():
                        if y in lst:
                            dct8[x] = self.accur[x]

                    dct8 = sort_dct_val(dct8)
                    name = vgf.dct_idx(dct8)
                    # if cap:
                    #     return dct5[name].capitalize()
                    # else:
                    return dct5[name]


    def agreement(self, lem2quant=0):  # 5610
        p (f'deciding which dictionary author is the most reliable')

        lst = [0, 1, 2, 3, 5, 6, 7]
        disg = {}
        disg2 = {}
        b = 0
        for k, v in self.col_dct.items():
            b += 1
            vgf.print_intervals(b, 1000, None, len(self.col_dct))
            if k == 'xerxes':
                bb = 8
            wrds = set()
            dct4 = {}
            dct5 = {}
            lemma = self.co_lemmas1a[k]
            beg = 0
            for i in lst:
                if len(v[i]) > 2:
                    if v[i][1][0].isupper():
                        beg = 1
                        break

            for i in lst:
                if i == 2:
                    bb = 8
                if len(v[i]) > 2:
                    name = v[i][0]
                    word = v[i][2]
                    word2 = self.jv.replace(word).lower()
                    end = len(word)
                    if reg(r'[aeiou]$', k):
                        end = -1
                    word1 = word[beg:end]
                    wrds.add(word1)
                    dct4[name] = word1
                    dct5[name] = word2

            if len(wrds) > 1:
                rword = self.lem2vquant(dct4, dct5, beg)
                lemma['macron'] = rword
            else:
                lemma['macron'] = word

            disg[k] = dct4
            disg2[k] = dct5

        self.disg2 = disg2



        dct = {}
        for k, v in self.co_lemmas1a.items():
            if not v['macron']:
                dct[k] = v
                p(k)
        if dct:
            p('error in macron')
            assert 0

        return



    def match_gen2stem(self):
        '''
        beginning with this method, i adopted a method
        where i tried to find the vowel lengths by separating words
        from their prefix and assuming that the prefix always had
        the same vowel length and that the main stem also always
        had the same vowel length.  This idea was discarded
        since it assumed too much.  The only way to really find out
        if a stem always had the same vowel length was to check it against
        the pedecerto database
        '''

        disg = self.disg2
        self.gen2stem = defaultdict(dict)
        miss = []
        self.gen2entry = defaultdict(dict)

        # dct = {}
        # for k, v in self.lem2part3.items():
        #     for x, y in v.items():
        #         if not y[0][2]:
        #             dct[k] = v
        #             break

        for k, v in self.lem2part3.items():
            obj = disg[k]
            for x, y in v.items():
                if len(y) > 2 and not y[2]:
                    self.gen2stem[k][x] = [0, lst]
                else:
                    lst = [remove_hats(z[2]) for z in y if z[2]]
                    stem = obj[x]
                    for z in y:
                        if z[2]:
                            g = remove_hats(z[2])
                            self.gen2entry[k][(x, g)] = z[1]
                        else:
                            self.gen2entry[k][(x, 0)] = 0
                    self.gen2stem[k][x] = [stem, lst]
        file = f'{fold}gen2stem'
        lst = [self.gen2stem, self.gen2entry, self.word2prefix, self.fake_prefixes, self.all_geninf]
        pi.save_pickle(lst, file, 1)
        return

    def auth_contr(self, debug=0):
        if debug:
            file = f'{fold}gen2stem'
            lst = pi.open_pickle(file, 1)
            self.gen2stem = lst[0]
            self.gen2entry = lst[1]
            self.word2prefix = lst[2]
            self.fake_prefixes = lst[3]
            self.all_geninf = lst[4]

        self.self_contr = defaultdict(dict)
        analyze = defaultdict(dict)
        stats = defaultdict(int)
        tot = defaultdict(int)
        dct = defaultdict(dict)
        for x, y in self.gen2stem.items():
            for k, v in y.items():
                if k != 'YO':
                    dct[x][k] = v
        self.gen2stem = dct
        ratio = {}
        p (f'now getting the genitive or perfect stem'
           f' from the dictionary entry')
        c =0
        for x, y in self.gen2stem.items():
            if self.kind != 'p':
                c += 1
                vgf.print_intervals(c,200,None, len(self.gen2stem))
            wrong = 0
            for k, v in y.items():
                if not v[1]:
                    bb = 8

                stem = v[0]
                stem2 = unidecode(stem)
                if k in ['Ge', 'PO']:
                    bb = 8
                for gn in v[1]:
                    tot[k] += 1
                    gn2 = unidecode(gn)
                    b = 0
                    found = 0
                    for t, z in zip(stem2, gn2):
                        ms = stem[b]
                        mg = gn[b]
                        if t == z:
                            if ms != mg:
                                self.self_contr[x][k] = v
                                s = self.gen2entry[x][(k, gn)]
                                analyze[x][k] = [gn, stem, s, 0]
                                stats[k] += 1
                                wrong += 1
                                found = 1
                                break
                        else:
                            break

                        b += 1
                    if not found:
                        s = self.gen2entry[x][(k, gn)]
                        analyze[x][k] = [gn, stem, s, 1]
            # if len(y) > 1:
            rat = int((wrong / len(y)) * 100)
            ratio[x] = rat
        ratio = sort_dct_val_rev(ratio)
        rat_ana = {}
        for k, v in ratio.items():
            rat_ana[k] = analyze[k]
        self.rat_ana = rat_ana
        # for k, v in tot.items():
        #     mis = stats[k]
        #     perc = int((mis / v) * 100)
        #     p(k, perc)

        return

    def ana_wo_prefix(self):
        self.wo_prefix = defaultdict(list)
        self.wo_prefix_fake = defaultdict(list)
        anom = {}
        anom2 = {}
        anom3 = {}
        pdct = {
            'col': 'con',
            'sur': 'sub',
            'suc': 'sus'
        }

        for k, v in self.rat_ana.items():
            if not v:
                anom3[k] = v
            else:
                if k == 'recido2':
                    bb = 8

                obj = vgf.dct_idx(v, 0, 'v')
                word = obj[0]
                word_nm = unidecode(word)
                head = obj[1]
                head_nm = unidecode(head)
                tpl = self.word2prefix.get(word_nm)
                pre = ""
                if tpl and k not in self.false_prefix:
                    pre = tpl[0]
                    main = tpl[1]
                    start = len(pre)
                    bod_head = head_nm[start:]
                    bod_pre = head_nm[:start]
                    if bod_pre != pre:
                        if pre not in pdct:
                            start = 0
                        anom[k] = v
                else:
                    start = 0
                    bod_head = head_nm[start:]
                    main = word_nm

                dct = {}
                dct1 = {}
                for auth, lst in v.items():
                    if k == 'recido2' and auth == 'Lw':
                        bb = 8

                    try:
                        word3 = lst[0][start:]
                        head3 = lst[1][start:]
                        lst1 = [word3, head3, lst[2], k, lst[3], pre]
                        dct[auth] = lst1
                    except IndexError:
                        word3 = lst[0]
                        head3 = lst[1]
                        dct1[auth] = [word3, head3, lst[2], k, lst[3], pre]
                if dct1:
                    anom2[main] = dct1
                if main in self.fake_prefixes:
                    self.wo_prefix_fake[main].append(dct)
                self.wo_prefix[main].append(dct)
        return

    def ana_wo_prefix2(self):
        self.ranking = {
            'GJ': 2,
            'PO': 6,
            'Lw': 4,
            'LS': 3,
            'GG': 4,
            'Ge': 5
        }
        freq = defaultdict(dict)
        singles = defaultdict(dict)
        fake_freq = defaultdict(dict)
        for x, y in self.wo_prefix.items():
            single = 0
            if len(y) == 1:
                single = 1
            for dct in y:
                for auth, lst in dct.items():
                    word = lst[0]
                    lemma = lst[3]
                    # prefix = lst[4]
                    freq[x].setdefault(word, []).append([auth] + lst)

        self.freq = freq
        return

    def count_freq(self):
        self.stats = {}
        self.tots = {}
        # for k, v in self.freq.items():
        for k, v in self.freq.items():
            dct1 = {
                'PO': 0,
                'Ge': 0,
                'GJ': 0,
                'all': 0
            }
            dct2 = defaultdict(dict)
            for x, y in v.items():
                for z in y:
                    auth = z[0]
                    if auth not in dct1:
                        dct1["all"] += 1
                        num = dct2['all'].get(x, 0)
                        dct2['all'][x] = num + 1
                    else:
                        dct1[auth] += 1
                        num = dct2[auth].get(x, 0)
                        dct2[auth][x] = num + 1

            for x, y in dct2.items():
                tot = dct1[x]
                if tot:
                    for s, t in y.items():
                        perc = int((t / tot) * 100)
                        y[s] = perc
                    y = sort_dct_val_rev(y)
                    for s, t in y.items():
                        y[s] = [t, tot]
                    dct2[x] = y

            self.stats[k] = [dct2, v]

        # self.stats = vgf.sort_dct_val_rev(self.stats)
        # ana = {}
        # for k, v in self.stats.items():
        #     ana[k] = [v, self.tots[k[0]]]
        return

    def choose_winners(self):
        self.wmac = {}
        self.undecided = {}
        self.error_spell = defaultdict(list)
        self.undecided2 = {}
        self.uwords = {}
        for k, v in self.stats.items():
            if k == 'cid':
                bb = 8
            scores = v[0]
            threshold = 7
            done = 0
            if len(scores) == 1 and 'GJ' in scores:
                word, perc, tot = self.output_dct(scores, 'GJ')
                if perc > 70:
                    self.wmac[k] = word
                else:
                    self.undecided2[k] = v
                    self.uwords[k] = word

            else:
                lst1 = ['PO', 'Ge']
                b = 0
                while 1:
                    for auth in lst1:
                        word, perc, tot = self.output_dct(scores, auth)
                        if tot > threshold:
                            if perc > 66:
                                done = 1

                            elif tot > 7 and auth == 'PO':
                                self.undecided[k] = v
                                self.uwords[k] = word
                                done = 1
                            if done: break
                    b += 1
                    if b > 1:
                        break
                    threshold = 3
                if not done:
                    und = 0
                    word, perc, tot = self.output_dct(scores, 'all')
                    wordp, percp, totp = self.output_dct(scores, 'PO')
                    wordg, percg, totg = self.output_dct(scores, 'Ge')
                    if word != wordp:
                        if totp < 3 and tot > 7 and perc > 80:
                            pass
                        elif totp > 1 and perc == 100:
                            word = wordp
                        elif percp == 100:
                            word = wordp
                        elif percg == 100:
                            word = wordg
                        elif perc > 66:
                            self.wmac[k] = word
                        else:
                            und = 1

                    else:
                        if percp == 100:
                            word = wordp
                        elif percg == 100:
                            word = wordg
                        elif perc > 66:
                            pass
                        else:
                            und = 1

                    if und:
                        if wordp:
                            word = wordp
                        elif wordg:
                            word = wordg

                        self.undecided2[k] = v
                        self.uwords[k] = word

            self.wmac[k] = word
            self.adjust_lemma(v[1], word)

        return

    def adjust_lemma(self, v, word):
        pres = set()
        lem2word = defaultdict(set)
        lem2pre = defaultdict(set)
        lem2bod = defaultdict(set)
        for s, lst in v.items():
            for z in lst:
                pre = z[6]
                pres.add(pre)
                lemma = z[4]

                if unidecode(word) != unidecode(z[1]):
                    self.error_spell[lemma].append([word, z])
                else:
                    bod = word
                    if lemma == 'patron':
                        bb = 8

                    lem2bod[lemma].add(bod)
                    if pre and pre[-1] == bod[0] and pre[:-1] in self.prefix2:
                        pass
                    elif pre:
                        pre = self.prefix.get(pre, '')
                        if not pre:
                            p(lemma, bod)

                    str1 = f'{pre}{bod}'
                    str1_wo_mac = unidecode(str1)
                    # self.macrons[str1_wo_mac][lemma] = str1
                    lem2word[lemma].add(str1)
            for k, t in lem2word.items():
                lem = self.co_lemmas1a[k]
                assert len(lem2bod[k]) == 1
                lem[self.form + 'm'] += list(t)
            # p (k, t)
        return

    def output_dct(self, scores, auth):
        po = scores.get(auth, {})
        if not po:
            return 0, 0, 0
        word, lst = vgf.dct_idx(po, 0, 'i')
        perc = lst[0]
        tot = lst[1]
        return word, perc, tot

    def alt_geninf(self):
        dct = {}
        dct1 = {}
        wrong_vowels = [
            '''
            the following words are roots of lemmas for which not
            enough info has been given regarding their vowel quantity
            '''
        ]
        for k, v in self.co_lemmas1a.items():
            st1 = set(v[f'{self.form}m'])
            dctm = {unidecode(x): x for x in st1}
            st2 = set(dctm.keys())
            ost = v[self.form]
            if k == 'abscondo':
                bb = 8

            ost1 = ost.split(',')
            ost1 = set(unidecode(x).lower() for x in ost1)
            miss = ost1 - st2
            if st1 and miss:
                if v[self.form] and ',' not in ost or not st2:
                    dct[k] = v
                else:
                    arb = list(st2)[0]
                    arb_wma = dctm[arb]
                    # arb_wma = self.macrons[arb][k]
                    hvow = re.sub(r'[^aeioyu]', "", arb)
                    hvowma = re.sub(r'[^aeioyuāēīōūȳ]', "", arb_wma)
                    for w in ost1:
                        if w not in dctm:
                            oword = w
                            for e, t in en(w):
                                if t in 'aeioyu':
                                    if t == hvow[0]:
                                        w = replace_at_i(e, w, hvowma[0])
                                        if len(hvowma) == 1:
                                            break
                                        hvowma = hvowma[1:]
                                        hvow = hvow[1:]
                                    else:
                                        wrong_vowels.append([oword, k])
                                        break
                            # self.macrons[oword][k] = w
                            v[f'{self.form}m'].append(w)

        to.from_lst2txt(wrong_vowels, f'{fold}stems_unknown_quant')

        return

    def final_test(self):
        for i in range(2):
            if i:
                dct = {}
            for x, y in self.co_lemmas1a.items():
                if y[self.form]:
                    lst = y[f'{self.form}m']
                    lst = [unidecode(x) for x in lst]
                    st = set(lst)
                    ost = set(y[self.form].split(','))
                    ost = set(w.lower() for w in ost)
                    if not st == ost:
                        miss = ost - st
                        for w in miss:
                            self.match_macrons(y, w)
                        if i:
                            dct[x] = y

        return

    def match_macrons(self, lemma, w):
        lem_wo_mac = lemma['lemma']
        lem_wmac = lemma['spell'].lower()
        lem_wmac = jv.replace(lem_wmac)
        hvow = re.sub(r'[^aeioyu]', "", lem_wo_mac)
        hvowma = re.sub(r'[^aeioyuāēīōūȳ]', "", lem_wmac)
        if hvow:
            for e, t in en(w):
                if t in 'aeioyu':
                    if t == hvow[0]:
                        w = replace_at_i(e, w, hvowma[0])
                        if len(hvowma) == 1:
                            break
                        hvowma = hvowma[1:]
                        hvow = hvow[1:]
                    else:
                        break
            lemma[f'{self.form}m'].append(w)


class bottom_most(step_one):
    def __init__(self):
        step_one.__init__(self)

    def begin(self, second=0):
        '''
        we run the begin function twice, the first time the form
        is set to geninf in which case we parse the geninf form
        the second time it is set to perf

        there is a function which divides the lemmas into each
        author's information

        '''

        self.second = second
        self.get_atts()
        if not self.second:
            self.normalize_perf()
            self.step1()
            self.alternate_spell()
            self.output(6)
            self.jv_rep()
            self.fix_ge()
            self.fix_lw()
            self.fix_yo()
            self.fix_po()
            self.jv_others()
            self.output(9)
            self.agreement()
        self.build_prefix_dct()
        self.begin_cl2()
        self.match_gen2stem()
        self.auth_contr()
        self.ana_wo_prefix()
        self.ana_wo_prefix2()
        self.count_freq()
        self.choose_winners()
        self.alt_geninf()
        self.final_test()
        if second:
            self.quick_fix()
            self.fdiph_fu()
            self.check_jv()
            self.output(2)





if eval(not_execute_on_import):
    if vgf.pycharm():
        args = [0, 'pcf', '', 'IX', '', 0, 0]
    else:
        args = vgf.get_arguments()

    ins = bottom_most()
    ins.begin()
    ins.kind = 'p'
    p('step two')
    ins.begin(1)






import os, sys

import Levenshtein as lvn
import sys, subprocess, zipfile
from PIL import ImageFont
from abbreviations import *


######### timers



def open_txt_file(file):
    if "." not in file[-4:]:
        file += '.txt'
    p = subprocess.call(['open', file])


def sort_lst_by_len(lst, long_first=0):
    dct = {x: len(x) for x in lst}
    if long_first:
        dct = sort_dct_val_rev(dct)
    else:
        dct = sort_dct_val(dct)

    return list(dct.keys())




def get_arguments():
    arguments = sys.argv
    try:
        arg1 = arguments[1]
    except:
        arg1 = ""
    try:
        arg2 = arguments[2]
    except:
        arg2 = ""
    try:
        arg3 = arguments[3]
    except:
        arg3 = ""
    try:
        arg4 = arguments[4]
    except:
        arg4 = ""
    try:
        arg5 = arguments[5]
    except:
        arg5 = ""

    return [1, arg1, arg2, arg3, arg4, arg5]


def del_last_empty_rw(lst):
    if lst:
        while type(lst[-1] == str) and not reg(r'\S', lst[-1]):
            del lst[-1]
    return lst



def pycharm():
    if 'kylefoley' in sys.argv[0] and len(sys.argv) == 1:
        return 1


def largest_member(dct, tie=0):
    # values must be integers or floats
    dct = sort_dct_val_rev(dct)
    if len(dct)>1 and tie:
        s = dct_idx(dct)
        t = dct_idx(dct,1,'v')
        if s == t:
            return (s,t)

    return dct_idx(dct)



def get_between_sline(s, ld='<', add_com=0):
    on = 0
    if ld == '<':
        rd = '>'
    elif ld == '[':
        rd = ']'
    elif ld == '{':
        rd = '}'
    t = ""
    for x in s:
        if x == ld:
            on = 1
        elif x == rd:
            on = 0
            if add_com:
                t += ','
            else:
                t += ' '
        elif on:
            t += x
    return t.strip()


def sort_dct_by_dct(to_be_ordered, orderer):
    dct2 = {}
    orderer = sort_dct_val_rev(orderer)
    for k,v in orderer.items():
        dct2[k] = to_be_ordered[k]
    return dct2





def convert_nums(x):
    for e, z in en(x):
        try:
            z = int(z)
            x[e] = z
        except ValueError:
            try:
                z = float(z)
                x[e] = z
            except ValueError:
                pass

    return x




def from_dct_sum2perc(dct):
    '''
    the dict must have integers as values
    this will convert the integers into percentages of a
    whole
    '''
    tot = 0
    for k,v in dct.items():
        tot += v
    dct1 = {}
    for k,v in dct.items():
        dct1[k] = int((v/tot)*100)
    return dct1





def inside(s, l, r):
    if s.count(l) != 1 or s.count(r) != 1:
        return ""
    t = lambda s, l, r: s[s.index(l) + 1:s.index(r)]
    return t(s,l,r)




def del_last_empty_lst(lst):
    try:
        while len(lst[-1]) == 1 and not reg(r'\S', lst[-1][0]):
            del lst[-1]
        return lst
    except:
        while not lst[-1]:
            del lst[-1]
        return lst




def outside(s, l, r):
    if s.count(l) != 1 or s.count(r) != 1:
        return ""

    if s[0] == l:
        return s[s.index(r) + 1:].strip()
    elif s[-1] == r:
        return s[:s.index(l):].strip()
    else:
        return ""



def strip_n_split(str1, char=" "):
    lst = str1.split(char)
    return [x.strip() for x in lst]




def dct_idx(dct, idx=0, kv='k'):
    if not dct:
        return {}

    if kv == 'k':
        return list(dct.keys())[idx]
    elif kv == 'i':
        return list(dct.items())[idx]
    else:
        return list(dct.values())[idx]



def err_idx(num, lst):
    if num > len(lst):
        return len(lst)
    return num




def end_list(lst, num):
    try:
        c = lst[num]
        return num
    except:
        return len(lst)



def beg_list(lst,num):
    try:
        c = lst[num]
        return num
    except:
        return 0


def get_text_size(text, font_size=18, font_name='Times New Roman'):
    font = ImageFont.truetype(font_name, font_size)
    size = font.getsize(text)
    return size

def printProgressBar (iteration, total, prefix = '', suffix = '', decimals = 1, length = 100, fill = '█', printEnd = "\r"):
    """
    Call in a loop to create terminal progress bar
    @params:
        iteration   - Required  : current iteration (Int)
        total       - Required  : total iterations (Int)
        prefix      - Optional  : prefix string (Str)
        suffix      - Optional  : suffix string (Str)
        decimals    - Optional  : positive number of decimals in percent complete (Int)
        length      - Optional  : character length of bar (Int)
        fill        - Optional  : bar fill character (Str)
        printEnd    - Optional  : end character (e.g. "\r", "\r\n") (Str)
    """
    percent = ("{0:." + str(decimals) + "f}").format(100 * (iteration / float(total)))
    filledLength = int(length * iteration // total)
    bar = fill * filledLength + '-' * (length - filledLength)
    print(f'\r{prefix} |{bar}| {percent}% {suffix}', end = printEnd)
    # Print New Line on Complete
    if iteration == total:
        print()


def print_intervals(number, interval, fork=None, total=0):
    if number > 0 and number % interval == 0 and number >= interval:
        if total:
            printProgressBar(number, total)
            # per = int((number / total) * 100)
            # number = f"{per}%"

        # if fork == None:
        #     p(number)
        # else:
        #     p(number)
        # else:
        #     p(f"fork {fork} - {number}")
        return


def unzip(path_to_zip_file, directory_to_extract_to):
    with zipfile.ZipFile(path_to_zip_file, 'r') as zip_ref:
        zip_ref.extractall(directory_to_extract_to)




from bglobals import *




# authors = ['LS', 'GG', 'GJ', 'Ge', 'FG', 'Lw', 'YO', 'PO', 'WW']


class colatinus():
    def __init__(self):
        '''
        this class uses the definitions of the online latin dictionary
        and combines them with the definitions of collatinus and
        prints all properties of each word to an excel spread sheet

        '''

    def begin(self):
        self.get_atts()
        self.corrupting_alternates()
        self.ad_hoc()
        self.simplify_models()
        self.new_mod_names()
        self.use_wrong_model()
        self.get_pos()
        self.get_short_defs()
        self.get_gender()
        self.get_word2defs()
        pi.save_pickle(self.co_lemmas2, f'{fold}co_lemmas3')
        return




    def get_atts(self):
        p ('opening pickles')
        self.co_lemmas2 = pi.open_pickle(f'{fold}co_lemmas2')
        self.col_dct = pi.open_pickle(f'{fold}col_dct')
        self.mmodels = pi.open_pickle(f'{fold}mmodels')
        self.lem_freq_rank = pi.open_pickle(f'{fold}lem_freq_rank')


    def corrupting_alternates(self):
        '''
        the following eliminates geninf and perf roots which derive from a variant
        spelling
        '''
        file = f'{fold}bogus_alt'
        lst5 = to.from_txt2lst(file)
        for x in lst5:
            if x.startswith('__'):
                lemma = x[2:]
                obj = self.co_lemmas2[lemma]
            if "*" in x:
                if x.startswith('g:'):
                    orig = obj['geninfmjv']
                else:
                    orig = obj['perfmjv']

                s = x[2:]
                l = s.split(',')
                m = [z for z in l if '*' not in z]
                mu = ','.join(m)
                # p (orig, mu)

                if x.startswith('g:'):
                    obj['geninfmjv'] = mu
                else:
                    obj['perfmjv'] = mu

        return




    def new_mod_names(self):
        file = f'{fold}modeles4'
        lst = to.from_txt2lst(file)
        self.omod2newmod = {}
        for x, y in zip(lst[:-1], lst[1:]):
            if x.startswith('modele:') and y.startswith('!;'):
                nmod = x[7:].strip()
                omod = y[2:].strip()
                self.omod2newmod[omod] = nmod
        pi.save_pickle(self.omod2newmod, f'{fold}omod2nmod')
        for k, v in self.co_lemmas2.items():
            v['model'] = self.omod2newmod.get(v['model'], v['model'])


    def use_wrong_model(self):
        file = f'{fold}wrong_model'
        file2 = f'{fold}wrong_model2'
        lst = to.from_txt2lst_tab_delim(file, 1, 1)
        lst1 = to.from_txt2lst_tab_delim(file2, 1, 1)

        for x in lst:
            xu = norm_str_jv(x[0])
            obj = self.co_lemmas2[xu]
            obj['model'] = x[1]
        for x in lst1:
            if x[0][0] != '/':
                xu = norm_str_jv(x[0])
                obj = self.co_lemmas2[xu]
                obj['model'] = 'sono'
        return

    def get_pos(self):
        '''
        the pos should be obtained in the w_convert_models
        do this later
        '''

        dct = {}
        research = 0
        if research:
            for e, x in en(list(self.mmodels.keys())):
                p(e, x)
        e = 0
        for k, v in self.mmodels.items():
            dct[e] = k
            if e == 0:
                v['pos'] = 'o'
            elif e < 58:
                v['pos'] = 'n'
            elif e < 83:
                v['pos'] = 'a'
            elif e < 107:
                v['pos'] = 'p'
            elif e < 136:
                v['pos'] = 'v'
            else:
                v['pos'] = 'd'
            x = self.mmodels[k]
            x['pos'] = v['pos']
            self.mmodels[k] = x
            e += 1

        st = set()
        for k, v in self.co_lemmas2.items():
            try:
                v['pos'] = self.mmodels[v['model']]['pos']
            except:
                st.add(v['model'])

        return



    def purge_chinese(self):
        nums = {
            5: 6,
            2: 4,
            1: 6,
            0: 6,
        }
        for k, v in self.col_dct.items():
            for aut, num in nums.items():
                try:
                    str1 = v[aut][num]
                    if any(w for w in str1 if ord(w) == 65279):
                        pass

                    elif any(w for w in str1 if ord(w) > 10000):
                        v[aut][num] = ''

                except:
                    pass

        return



    def get_short_defs(self):
        '''
        169 words are found in the short defs which
        do not have a lemma but most are heteronyms
        '''

        lst = to.from_txt2lst(f'{fold}lemmes_en_mean')
        self.lemmes_en_mean = {}
        wrong = []
        miss2 = {}

        for x in lst:
            if x and x[0] != '!':
                if x.count(':') == 1:
                    y = x.split(":")
                    key = y[0]
                    key = norm_str_jv(key)
                    # itm = self.by_class.get(key)
                    itm = self.co_lemmas2.get(key)
                    if itm:
                        itm['def_co'] = y[1]

                else:
                    wrong.append(x)
        miss2 = vgf.sort_dct_val(miss2)
        return

    def ad_hoc(self):
        file = f"{fold}bad_macrons"
        dct4 = to.from_txt2dct_1d(file)
        for k, v in dct4.items():
            self.co_lemmas2[k]['macronjv'] = v

        del self.co_lemmas2['cn']
        dct = {
            'nereides': 'nereid',
            'aether2': 'aetheris',
            'philochares2': 'philocharetis',
            'sanguinans': 'sangụinant'
        }
        for k, v in dct.items():
            obj = self.co_lemmas2[k]
            obj['geninfmjv'] = v
        self.co_lemmas2['concustodio']['perfmjv'] = 'concustōdi'



    def simplify_models(self):
        lst = ['lexicon', 'syl', 'spell', 'jv', 'macronjv',
               'perfm', 'geninfm', 'macron', 'geninfmjv', 'perfmjv',
               'operf', 'ogeninf']

        for k, v in self.co_lemmas2.items():
            v['def_co'] = ''
            if 'perfmjv' in v:
                v['perf'] = v['perfmjv']
            if 'geninfmjv' in v:
                v['geninf'] = v['geninfmjv']

            del v['quantity']
            v['lemma'] = v['macronjv']
            # if v['lemma'][-1].isdigit():
            #     p (v['lemma'])
            #     v['lemma'] = v['lemma'][:-1]

            for x in lst:
                if x in v:
                    del v[x]


        return




    def get_gender(self):
        dct = {
            0: 4,
            1: 4,
            2: 3,
            3: 3,
            5: 4,
            6: 6
        }
        errors = defaultdict(dict)
        col_errors = {}
        b = 0
        c = 0
        d = 0
        f = 0
        p (f'now getting the gender of the nouns')
        self.co_lemmas2['bos']['model']='miles'
        self.co_lemmas2['bos']['pos']='n'
        for k, v in self.co_lemmas2.items():
            b += 1
            vgf.print_intervals(b, 1000, None, len(self.co_lemmas2))
            if v['pos'] == 'n':
                itm = self.col_dct.get(k)
                if not itm:
                    col_errors[k]=v
                else:
                    st = set()
                    if k == 'labellum':
                        bb = 8
                    lst = []
                    for x, y in dct.items():
                        found = 1
                        try:
                            s = itm[x][y]
                        except:
                            found = 0
                        if found:
                            ostr = s
                            if hl(s):
                                found = 0
                                for g in ['m', 'n', 'f']:
                                    g1 = f' {g} '
                                    g2 = f' {g}.'
                                    g3 = f'{g}.'
                                    if g1 in s or g2 in s \
                                            or g3 == s:
                                        found = 1
                                        if not st or g not in st:
                                            lst.append(g)
                                        st.add(g)
                                        lst.append(g)

                                if not found:
                                    errors[k][x] = ostr
                                else:
                                    c += 1

                    if st:
                        lst5 = list(st)
                        lst5.sort()
                        v['gender'] = ''.join(lst5)
                        f += 1
                    else:
                        d += 1

        return


    def get_word2defs(self):
        nums = {
            5: 6,
            0: 6,
            2: 4,
            1: 6,
            3: 4,
        }
        self.no_def = set()
        errors = defaultdict(dict)
        for k, cl in self.co_lemmas2.items():
            v = self.col_dct[k]
            found = 1 if cl['def_co'] else 0
            for aut, num in nums.items():
                auth = authors[aut]
                s = f'def_{auth.lower()}'
                cl[s] = ""

            for aut, num in nums.items():
                if aut in [1, 3] and found:
                    pass
                else:
                    try:
                        if len(v[aut]) > num:
                            defn = v[aut][num]
                            if defn:
                                if not ord(defn[0]) > 30_000:
                                    auth = authors[aut]
                                    found = 1
                                    s = f'def_{auth.lower()}'
                                    cl[s] = defn
                    except:
                        errors[k][aut] = self.col_dct[k][num]
            if not found:
                self.no_def.add(k)

        dct7 = {}
        for k in self.no_def:
            j = cut_num(k)
            rank = self.lem_freq_rank.get(j, 99_999)
            dct7[k] = rank
        dct7 = sort_dct_val(dct7)

        return





if eval(not_execute_on_import):
    if vgf.pycharm():
        args = [0, '', '', 'IX', '', 0, 0]
    else:
        args = vgf.get_arguments()

    ins = colatinus()
    ins.begin()





import os
from bglobals import *




'''
1. convert lasla to text
1.4 stems A
1.5 memorization table
2. macronizer without lasla
3. parsed OLD
4. scan poetry
5. search engine
6. add english



occido3 = occidoō
sallio,salio = to salt, is missing though it might have been deleted because it has the same form
in salio2 it gets conjugated as saliio

about 10 of the lemmas in lasla are capitalized
unabbreviate words
enclose aux in ( )
unenclose non aux
remove < >
remove space words
some lines have two spaces
put aux on a different line
add ft.sg and locative adjectives
add more lemmas
change -um adjectives to -us
handle abbreviated s
handle bogus pos
perhaps replace numbers with spoken
handle alternative spellings
handle greek words
insert punctuation
research disagreement in conjugation
'''



def help():
    p ("""
    The purpose of this module is to normalize the lemmas, and the parts of speech (pos)
    of the LASLA database. We are not here interested in normalizing the words.
    We also take the first steps in developing a system such that the LASLA database
    can be easily converted into a readable text document.  First, we must distinguish words
    which are to be printed when converting to text and words which do not.
    The main object we output in this module is the 'auth2work' object.  This is dictionary of lists
    and in each list is contained another list each containing most of the time one word.
    This list has the following dimensions:
    
    0: here we write down certain categories and how the list should be handled
    
    "": the most common category, simply print the word
    !: do not print the word, in dimension 4 we will state why
    @: print the word, but do not use it for gathering statistics.
        these are words whose pos was not put in a standard form
    
    1: this dimension contains the lemma.  If a word is proper and a noun
        then the lemma ends with 9.  If the word is proper and a adjective
        then the lemma ends with 8.
        The index numbers of the lemmas in the original were found on the word
        not the lemma but this was changed here to putting the index numbers on the lemmas.
        lemmas are also put in all lowercase and j and v are placed with i and u respectively.
        
    2: this dimension contains the word.  All words are put in lowercase.  if they are
    proper then that info gets put on the lemmas as mentioned earlier.
        if the word is greek is became garbled in lasla and here is changed
        to simply [GREEK]
        
    3: the numbering of the latin text. nothing is changed here.
    
    4: the pos gets converted into something human readable. 
    we also use the pos found in the collatinus database which contains
    416 different pos.  lasla has about 10 other pos which are not contained in collatinus.
    the adjectives were also labeled with three codes which are not found in collatinus.
    these are
    C: masculine, feminine or neuter
    B: masculine, feminine
    X: masculine or neuter
    
    these are put in capital letters so as to be able to easily convert
    them to the official pos
    
    
    when an pos is missing an elemment we label that element with captal letters
    as follows
    
        gender = 'G'
        person = 'P'
        case = 'A'
        mood = 'M'
        voice = 'V'
        tense = 'T'
        number = 'N'
        degree = 'D'
    
    None of the converted collatinus pos have capital letters so
    if a pos has capital letters you know that something is wrong with it
    
    On a few occasions lasla used the number 0 although it is not written
    down in their guide book what 0 means.  
    
    
    
    """)











class parse_words:
    def __init__(self):
        pass

    def get_atts_pw(self):
        self.wst1 = set()
        self.wst2 = set()
        self.all_lines2 = set()
        self.odd_words = set()
        self.odd_words2 = set()
        self.odd_words3 = set()
        self.all_words_pw = set()
        self.bad_words = set()
        self.greek = set()
        self.tlemma = ''
        self.tword = ''
        self.lem_num = ''
        self.capitals = set()
        self.short = set()
        self.toword = ''
        self.pos3 = ''

        pass

    def del_attribs(self):
        self.all_atts = ['wst1', 'wst2', 'all_lines2',
                         'all_words_pw',
                         'bad_words', 'greek', 'tlemma', 'toword',
                         'pos3', 'capitals', 'short', 'lem_num']
        o = ['odd_words2', 'odd_words', 'odd_words3']
        for x in self.all_atts:
            delattr(self, x)

        more_atts = ['lem2forms']
        for x in more_atts:
            try:
                delattr(self, x)
            except:
                pass

    def parse_words_fu(self):

        for line in self.all_lines:
            self.tlemma = line[1]
            self.tword = line[0]
            self.pos3 = line[2]
            self.toword = self.tword

            if len(self.tword) > 1:
                if self.tword[0] == '$':
                    self.greek.add(self.tword)
                else:
                    self.parse_words_fu2()

                    if reg(r'[^a-zA-Z]', self.tword):
                        pass
                    else:
                        wordu = norm_str_jv(self.tword)
                        lemmau = norm_str_jv(self.tlemma)
                        if wordu in self.macronizer_new:
                            self.all_lines2.add((lemmau, wordu, self.pos3))
                            self.all_words_pw.add(wordu)
                        else:
                            self.bad_words.add(wordu)

        # col_words = set(self.macronizer_new.keys())
        # no_col = all_words - col_words
        # p (len(no_col))
        pi.save_pickle(self.all_lines2, f'{fold}lasla_lines2', 1)

        return

    def parse_words_pre(self):
        if len(self.tword) > 1:
            if self.tword[0] == '$':
                self.greek.add(self.tword)
            else:
                self.parse_words_fu2()

    def parse_words_fu2(self):
        self.lem_num = ""
        if "(" in self.tword:
            self.line2paren[(self.work, self.line)] = self.tword
            inside = vgf.inside(self.tword, '(', ')')
            if inside in self.sum2pos_dct:
                self.sum_word = inside
                self.sum_paren = 1

            self.tword = re.sub(r'\(.*\)', '', self.tword)
            self.has_paren = 1

        if not hl(self.tword):
            self.add2errors('odd_word2', self.tword)
        else:
            if self.tword[0].isdigit():
                self.lem_num = self.tword[0]
                self.tword = self.tword[1:]
                self.tword = self.tword.strip()




            elif self.is_proper():
                pass

            elif self.tword[0].isupper():
                self.tlemma = self.tlemma.capitalize()

            if len(self.tword) > 1 and self.tword[0] == ' ':
                self.odd_words3.add(self.toword)
            else:
                if reg(r'[^a-zA-Z]', self.tword):
                    self.odd_words.add(self.tword)

    def is_proper(self):
        if self.tword == 'Areos Pagos':
            bb = 8

        if len(self.tword) > 1 and self.tword[0] in ['N', 'A']:
            if self.tword[1].isupper():
                if len(self.tword) > 2:
                    if self.tword[2].isupper():
                        self.wst1.add(self.tword)
                    else:
                        beg = self.tword[0]
                        self.lem_num = '9' if beg == 'N' else '8'
                        self.tword = self.tword[1:]
            else:
                if len(self.tword) < 3:
                    self.short.add(self.tword)
                else:
                    if self.tword[:2].lower() == self.tlemma[:2].lower():
                        self.capitals.add((self.tword, self.tlemma))
                    else:
                        beg = self.tword[0]
                        self.lem_num = '9' if beg == 'N' else '8'
                        self.tword = self.tword[1:]


def is_proper2(word):
    if len(word) > 1 and word[0] in ['N', 'A']:
        if word[1].isupper():
            if len(word) > 2:
                word = word[1:]

    return word


class convert2txt(parse_words):
    def __init__(self):
        parse_words.__init__(self)

    def begin_ct(self):
        self.kind = ''
        self.get_atts_pw()
        self.get_atts()
        self.fix_sum()

        if not self.kind in ['small', 'debug']:
            self.ad_hoc()
            self.lasla_typos()
        self.main_loop()
        self.del_attribs()
        self.output(1)

        return

    def begin1(self):
        self.build_db()
        self.all_lasla_lemmas()
        self.parse_words_fu()

    def begin3(self):
        self.review()
        self.normal_words()
        self.review_errors2()
        self.review_errors()
        self.output(2)

    def get_atts(self):
        # self.kind = 'small'
        if not public:
            self.auth2work = pi.open_pickle(f'{fold}lasla_db', 1)
        else:
            self.rename_db3()
        if self.kind in ['small','debug']:
            self.auth2work = vgf.split_dct(self.auth2work, 0, 1)
            self.auth2work_db = copy.deepcopy(self.auth2work)
        self.final_pos = pi.open_pickle(f'{fold}final_pos', 1)
        self.final_pos_rv = {v: k for k, v in self.final_pos.items()}
        # self.lem2forms = pi.open_pickle(f'{sfold}lem2forms_wonum', 1)
        # self.macronizer_new = pi.open_pickle(f'{sfold}macronizer_new', 1)
        # self.all_lines = pi.open_pickle(f'{sfold}lasla_lines',1)
        # self.all_lines = pi.open_pickle(f'{sfold}lasla_lines2', 1)
        self.verb_dcts()
        self.errors = defaultdict(list)
        self.verb_dcts2()
        self.verbs_forms = defaultdict(set)
        self.adj_forms = defaultdict(set)
        self.lemma2decl = defaultdict(dict)
        self.pos_ct = defaultdict(int)
        self.bogus_pos = defaultdict(list)
        self.done = 0
        self.has_paren = 1
        self.sum_paren = 0
        self.true_pos = defaultdict(list)
        self.true_pos2 = defaultdict(list)
        self.no_case = defaultdict(list)
        self.no_gender = defaultdict(list)
        self.lem2info = defaultdict(list)
        self.all_lems = set()
        self.sum_errors = defaultdict(list)
        self.sum2pos_dct = pi.open_pickle(f'{fold}sum2pos', 1)
        self.sum_word = ""
        self.owords = defaultdict(set)
        self.lafreq = defaultdict(int)
        self.strange_words = set()
        self.sum_lemmas = set()
        self.adjacent2 = defaultdict(set)
        self.anomalies = defaultdict(set)
        self.line2paren = {}
        # self.anomalies2 = pi.open_pickle(f'{sfold}anomalies',1)

        return

    def output(self, kind=1):
        if kind == 3:
            pi.save_pickle(self.auth2work, f'{fold}lasla_db2', 1)
        elif kind:
            pi.save_pickle(self.auth2work, f'{fold}lasla_db2', 1)
            pi.save_pickle(self.bogus_pos, f'{fold}bogus_pos', 1)
            pi.save_pickle(self.true_pos2, f'{fold}bogus_pos2', 1)
            pi.save_pickle(self.errors, f'{fold}lasla_errors', 1)
            pi.save_pickle(self.anomalies, f'{fold}lasla_anomalies', 1)
            pi.save_pickle(self.lafreq, f'{fold}las_freq', 1)
            pi.save_pickle(self.lemma2decl, f'{fold}lemma2decl', 1)

        return

    def fix_sum(self):
        self.sum2pos_dct['iri'] = ['if.fu.pa']
        self.sum2pos_dct['siet'] = ['3.pr.sj.ac']
        self.sum2pos_dct['siem'] = ['4.pr.sj.ac']
        self.sum2pos_dct['sies'] = ['2.pr.sj.ac']
        self.sum2pos_dct['sient'] = ['6.pr.sj.ac']

    def rename_db1(self):
        lst = []
        for k,v in self.auth2work.items():
            for x,y in v.items():
                lst.append([k,len(y),x,x])
        file = f'{fold}rename_lasla'
        to.from_lst2txt_tab_delim(lst, file)
        vgf.open_txt_file(file)

    def rename_db2(self):
        lst = []
        lst2 = []
        for k,v in self.auth2work.items():
            dct={}
            for x,y in v.items():
                xu = x.split('_')
                xu = xu[1]
                lst.append([xu,f'{k},{x}'])
                dct[x] = y
            file = f'{lafold2}{k}'
            pi.save_pickle(dct,file)
        file2 = f'{fold}rename_lasla2'
        to.from_lst2txt_tab_delim( lst,file2)
        return

    def rename_db3(self):
        # file2 = f'{fold}rename_lasla2'
        # lst = to.from_txt2lst_tab_delim(file2)
        # dct = {}
        # for x in lst:
        #     k = x[0]
        #     auth, work = x[1].split(',')
        #     dct[auth]
        self.auth2work = {}
        file = f'{lafold}'
        for l in os.listdir(file):
            if not l[0]=='.':
                auth = l[:-4]
                file3 = file + l
                obj = pi.open_pickle(file3)
                self.auth2work[auth] = obj

        return



    def build_db(self):
        bpn = f'{lafold}bpn/'
        self.auth2work = defaultdict(dict)
        b = 0
        p (f"""
        now looping through the lasla files and putting
        them into a python user-friendly format
""")

        tot = len(os.listdir(bpn))
        for x in os.listdir(bpn):
            b += 1
            vgf.print_intervals(b, 5, None,tot)
            if x[0] != '.':
                auth = x[:x.index('_')]
                work = x[x.index('_') + 1:-4]
                lst = to.open_any(bpn + x)
                lst1 = lst.split('\n')
                words = []
                for z in lst1:
                    lst2 = z.split('  ')
                    lst3 = [w.strip() for w in lst2 if hl(w)]
                    if len(lst3) > 4:
                        lst4 = lst3[:3]
                        idx = z.index(f' {lst4[2]} ')
                        final = z[idx + len(lst4[2]) + 2:]
                        final = final.lstrip()
                        lst4.append(final)
                    else:
                        lst4 = lst3
                    lst4.insert(0, z)
                    words.append(lst4)
                self.auth2work[auth][work] = words
        pi.save_pickle(self.auth2work, f'{fold}lasla_db', 1)
        return

    def build_apn(self):
        apn = f'{lafold}apn/'
        self.auth2work_ap = defaultdict(dict)
        b = 0
        for x in os.listdir(apn):
            b += 1
            vgf.print_intervals(b, 5)
            if x[0] != '.':
                auth = x[:3]
                work = x[3:]
                lst = to.open_any(apn + x)
                lst1 = lst.split('\n')
                words = []
                for z in lst1:
                    lst2 = z.split('  ')
                    lst3 = [w.strip() for w in lst2 if hl(w)]
                    if len(lst3) > 4:
                        lst4 = lst3[:3]
                        idx = z.index(f' {lst4[2]} ')
                        final = z[idx + len(lst4[2]) + 2:]
                        final = final.lstrip()
                        lst4.append(final)
                    else:
                        lst4 = lst3
                    lst4.insert(0, z)
                    words.append(lst4)
                self.auth2work_ap[auth][work] = words

    def analyze(self):
        lstb = self.auth2work['Cato']['CatoAgricultura_CatAgr']
        lsta = self.auth2work_ap['Cat']['Agric.APN']
        lst1 = []
        sta = set()
        stb = set()
        for a, b in zip(lsta, lstb):
            if len(a) > 4 and len(b) > 4:
                if len(b[4]) == 13:
                    b[4] = b[4][:-1]
                sa = a[4].strip()
                sb = b[4].strip()
                sta.add(sa)
                stb.add(sb)
                if sa != sb:
                    p(sa, sb)
                # lst1.append([a[4],b[4]])
        stc = sta - stb

    def all_lasla_lemmas(self):
        self.all_lemmas = set()
        self.all_words = set()
        self.all_lines = set()

        errors = []
        b = 0
        for auth, works in self.auth2work.items():
            for name, work in works.items():
                b += 1
                vgf.print_intervals(b, 5)
                for info in work:
                    if not len(info) > 1:
                        errors.append(info)
                    else:
                        if len(info[1]) > 8:
                            word = info[1][8:]
                            self.lemma = norm_str_jv(word)
                            self.word = info[2]
                            self.all_words.add(self.word)
                            self.all_lemmas.add(self.lemma)
                            if len(info) > 4:
                                pos = info[4]
                                pos2 = self.elim_edit(pos)
                                self.all_lines.add((self.word, self.lemma, pos2))
                            else:
                                errors.append(info)

                        else:
                            errors.append(info)
        pi.save_pickle(self.all_lines, f'{fold}lasla_lines', 1)

        return

    def elim_edit(self, num2):
        if len(num2) > 9:

            if len(num2) > 12:
                s = num2[:9] + '   ' + num2[12]
                return s
            else:
                s = num2[:9]
                return s

        else:
            if num2[0] == 'B':
                bb = 8

            return num2

    def ad_hoc(self):
        self.auth2work['Plautus']['Captiui_PlCapt'][2069] = ['*', '01234567Thensaurochrysonicochrysides',
                                                             'Thensaurochrysonicochrysides', 285, 'ns']
        self.auth2work['Plautus']['Captiui_PlCapt'][5173] = ['*', '01234567Thensaurochrysonicochrysides',
                                                             'Thensaurochrysonicochrysides', 285, 'ns']

    def lasla_typos(self):
        lst = to.from_txt2lst_tab_delim(f'{fold}lasla_typos')
        lst1 = to.from_txt2lst_tab_delim(f'{fold}lasla_typos2', 0, 1)
        lst += lst1
        for x in lst:
            auth = x[0]
            work = x[1]
            line = x[2]
            idx = x[3]
            s = x[4]
            try:
                orig = self.auth2work[auth][work][line][idx]
                self.auth2work[auth][work][line][idx] = s
            except:
                self.auth2work[auth][work][line].append(s)

            # p (f'replaced {orig}  with {s}')
        return

    def main_loop(self):
        b = 0
        self.num_lines = 0
        self.insert = {}
        self.infos = []
        self.adjacent = set()
        self.ninfo = ''
        self.lword = ''
        self.enclits = set()
        self.adjacent3 = 1
        self.paren2loc = {}
        self.delay_sum = 0
        p (f"""
        now looping through the lasla database
        and normalizing it
""")

        for self.auth, works in self.auth2work.items():
            for self.work, self.infos in works.items():
                b += 1
                vgf.print_intervals(b,5,None,236)

                if self.kind == 'debug':
                    self.work = 'PhilippicaOratio_CicPhi08'
                    self.auth = 'Cicero'
                    self.infos = self.auth2work[self.auth][self.work]
                for self.line, info in en(self.infos):
                    if self.line == 4:
                        bb = 8
                    if self.work == '' and self.line == 2692:
                        bb = 8
                    self.num_lines += 1
                    self.info = info
                    self.cpos = ''
                    self.elsum = 0
                    self.backwards_sum = 0
                    self.sum_paren = 0
                    try:
                        self.ninfo = self.infos[self.line + 1]
                    except:
                        self.ninfo = ''
                    self.special = 0
                    self.main_loop2(info)


                if b in [50, 100]:
                    bb = 8
            # if b > 50:
            #     break

        return

    def temp17(self):
        lst5 = []
        for e, x in en(self.errors['bad_sum']):
            if e < 9 or e > 37:
                lst5.append([x[1], x[2], x[3], '2', x[0]])
        for e, x in en(self.errors['bad_sum']):
            if e < 9 or e > 37:
                lst5.append([x[1], x[2], x[3], 4, '0'])

        file = f'{fold}temp_typos'
        to.from_lst2txt_tab_delim(lst5, file)
        vgf.open_txt_file(file)

    def temp18(self):
        st = set()
        for self.auth, works in self.auth2work.items():

            for self.work, self.infos in works.items():
                # self.work = 'DeOfficiis_CicOffic1'
                # self.auth = 'Cicero'
                # self.infos = self.auth2work[self.auth][self.work]
                for self.line, info in en(self.infos):
                    if '&' in info[1] and info[0] != '!':
                        p(self.auth, self.work, self.line)
                        break

    def main_loop2(self, info):
        '''
        the lasla database has the following types of lines
        a standard line has a recoginizable pos and has no strange characters in the word

        '''

        if len(info) == 1:
            self.tword = ''
            self.tlemma = ''
            self.pos = ''
            self.add2pos('_1l')

        elif len(info) != 5 or info[2][0] == '$':
            if info[2][0] == '$':
                self.tword = 'GREEK'
                self.add2pos('_gr')
            else:
                self.add2pos('_4')
                self.add2errors('_4', self.tword)

        elif info[0] in ['*', '!']:
            pass

        else:
            word1 = ''
            if self.kind == 'set':
                self.word = info[1]
                self.lemma = info[0]
                num2 = info[2]
                self.num2 = num2
            else:
                self.cpos = 0
                self.has_paren = 0
                self.raw = info[0]
                self.tlemma = self.parse_lemma(info[1])
                self.word = info[2]
                self.idx = info[3]
                self.tword = self.word
                self.toword = self.tword
                self.parse_words_pre()
                self.tlemma = self.tlemma + self.lem_num



            if self.tword and self.tword[0] == '$':
                self.cpos = 'gr'
                self.add2pos('_gr')

            else:

                if reg(r'^#[A-Z]', self.info[4]):
                    self.info[4] = self.info[4][1:]
                    self.special = 1

                self.mpos = info[4][0]

                if reg(r'[A-Z]', self.mpos):
                    self.convert_pos3(info, self.mpos)

                elif self.tlemma == '#' or self.mpos in ['#', '0'] or \
                        self.tlemma in ['_sum2', '_eo2']:

                    if self.tword == 'dignam':
                        bb = 8

                    if self.tlemma.startswith('_sum') and self.tword.endswith('st') and \
                            len(self.tword) > 3 and '<' not in self.tword:
                        self.sum_elision()


                    elif not ' ' in self.tword:
                        """
                        these are words that were not understood or are in lacunae
                        if they are consecutive we might surround them with marks
                        if they are genuine words we will macronize them
                        """

                        self.add2pos('_nsp')



                    elif self.proper_space():
                        if self.tword in ['reos Pagos', 'oui Comi']:
                            self.add2errors('bad_name', self.tword)
                        self.add2pos('_psp')


                    elif self.tword.count('>') == 1 and self.separable():
                        pass
                    elif self.delay_sum:
                        self.auth2work[self.auth][self.work][self.line][0] = '!'
                        self.auth2work[self.auth][self.work][self.line][4] = 'ds'
                        self.delay_sum = 0
                    else:
                        if self.ninfo and len(self.ninfo) > 2:
                            onword = self.ninfo[2]
                        else:
                            onword = ''

                        if self.line > 0:
                            try:
                                olword = self.infos[self.line - 1][2]
                                lword = re.sub(r'[0-9]', '', olword).strip()
                                lword = is_proper2(lword)
                                lword = re.sub(r'[<>\(\)]', '', lword).strip()
                                self.lword = re.sub(r'\s{2,}', ' ', lword)
                            except:
                                self.lword = ''
                        else:
                            self.lword = ''

                        onword2 = self.lasla_match(onword)
                        word1 = self.lasla_match(self.tword)

                        if word1 == onword2:
                            '''
                            for the sum, eo words lasla assigned a pos on the following or previous word
                            where the pos of sum and its participle can be deduced
                            for the other words in the future after they are macronized combine them
                            since that makes it easier to find them as a lemma,
                             some words cannot be easily combined like tanto opere
                            for now
                            if words on consecutive lines are both the same then simply delete
                                one of the lines, combine and put on one line
                            
                            '''

                            if self.tlemma in ['_sum2', '_eo2']:
                                self.add2pos('_yf')
                            else:
                                self.is_compound(1)
                                self.add2pos('_rf')

                        elif word1 == self.lword:
                            if self.tlemma in ['_sum2', '_eo2']:
                                self.add2errors('sum4', self.tword)

                                self.add2pos('_yp')
                            else:
                                self.is_compound(-1)
                                self.add2pos('_rp')

                        elif self.tlemma in ['_sum2', '_eo2']:
                            '''
                            words should not be going in here
                            '''

                            self.add2errors('bad_sum', self.tword)
                            self.add2pos('_w')

                        else:
                            '''
                            these words have spaces but should not have them
                            
                            these words have no pos, if their macronization is
                            unambiguous then macronize them, otherwise use some notation
                            to indicate that there is a problem with them, assign
                            ambig pos if possible
                            in some case there is an error where the two words are truly separated
                            but < > was not put around properly
                            
                            if one of the words is a sum word then the words are
                            divided and put on seperate lines and a pos assigned
                            where possible
                            
                            '''
                            self.add2pos('_n')


                else:
                    self.add2pos('_b')
                    self.add2errors('bad_pos', self.tword)

                self.lword = word1 if word1 else self.tword

    def is_compound(self, idx):
        if self.tword.count(' ') > 1 and idx > 0:
            nw = self.auth2work[self.auth][self.work][self.line + 1][2]
            nw = self.lasla_match(nw)
            if nw == self.tword:
                self.auth2work[self.auth][self.work][self.line + 1][0] = '!'

    def sum_elision(self):
        self.add2pos('_sel')
        self.add2errors('_sel', self.tword)


    def proper_space(self):
        try:
            nword = self.infos[self.line + 1][2]
        except:
            nword = ''
        try:
            lword = self.infos[self.line - 1][2]
        except:
            lword = ''

        if nword and nword[0] in ['N', 'A'] and self.word == nword[1:]:
            return 1
        elif lword and lword[0] in ['N', 'A'] and self.word == lword[1:]:
            return 1

    def add2errors(self, id, word):
        self.errors[id].append([word, self.auth, self.work, self.line])

    def lasla_match(self, word):
        onword2 = re.sub(r'[0-9]', '', word).strip()
        onword2 = is_proper2(onword2)
        onword2 = re.sub(r'[<>]', '', onword2).strip()
        onword2 = re.sub(r'\s{2,}', ' ', onword2)
        return onword2

    def convert_pos3(self, info, mpos):
        if mpos == 'A':
            self.nouns(info)
        elif mpos == 'B':
            self.verbs(info)
        elif mpos == 'C':
            self.adj(info)
        elif mpos == 'M':
            self.adv(info)
        else:
            self.other(info)
            pass

    def parse_lemma(self, s):
        lemma = s[8:]
        lemma = norm_str_jv(lemma)
        return lemma

    def iri(self, cpos):
        if self.sum_word in ['iri', 'eiri'] and 'fu.pa' in cpos:
            return 1

    def build_lem2info(self):
        for k, v in self.auth2work.items():
            if v[0]:
                if v[2] not in ['gr', 's']:
                    self.lem2info[v[0]].append([v[1], v[2]])
                    self.lafreq[v] += 1

    def add2pos(self, cpos):
        '''
        seperable well-formed words now have a _ on the lemma

        '''

        self.cpos = cpos
        if '(' in self.tword and cpos != '_gr':
            self.add2errors('has_paren', self.tword)
            bb = 8

        if '&' in self.tlemma or reg(r'[0-9]', self.tlemma[:-1]):
            bb = 8

        lst = ['', self.tlemma.lower(), self.tword, self.idx, self.cpos]
        tpl = (self.tlemma.lower(),self.tword, self.cpos)
        self.auth2work[self.auth][self.work][self.line] = lst


        if "_" in cpos:
            '''
            _1l = blank line
            _4 = list does not have 5 elements (fix)
            _nsp = pos in #, 0 but not space in the word
            _y sum word were one line = next line
            _rf, _rp has space and word = next word (no longer anomalous)
            _w space in word and lemma is _sum2
            _n hash_words
            _sp seperable but ill formed
            _s seperable well formed 
            _npsp proper nouns with a space which equal the following word           
            '''
            if cpos.startswith('_r') or cpos in ['_ss', '_n', '_1l']:
                lst[0] = '!'
            elif cpos.startswith('_nsp'):
                lst[0] = '?'
            self.anomalies[cpos].add(self.tword)

        elif  reg(r'[\s&\*\)]',cpos):

            '''
            these are parts of speech which lasla left incomplete, ie,
            did not state the case or tense, etc
            '''
            lst[0] = '@'
            self.add2errors('miss pos', self.tword)


        elif cpos:

            if cpos == 'gd.pb':
                cpos = 'gv.pmb'
            elif cpos == 'gd.pd':
                cpos = 'gv.pmd'

            if cpos.startswith('gd.') and cpos not in self.final_pos_rv:
                bb=8


            if cpos in self.final_pos_rv or self.iri(cpos):
                if self.iri(cpos):
                    pass
                else:
                    self.pos_ct[self.final_pos_rv[cpos]] += 1
                self.true_pos[cpos].append([self.tlemma, self.tword])
                self.lafreq[tpl] += 1


            elif reg(r'[A-Z]',cpos):
                self.true_pos2[cpos].append([self.tlemma, self.tword])


            else:
                if cpos in ['m']:
                    pass
                else:
                    self.add2errors('bad_pos' ,cpos)
                    self.bogus_pos[cpos].append([self.auth, self.work, self.line, self.tlemma, self.tword])

            self.cpos = cpos

    def anomaly_test(self):
        lst = ['_yf', '_yp']
        for y in lst:
            for x in self.anomalies[y]:
                if not x.count('>') == 1 or not x.count('<') == 1:
                    p(x)

    def missing_pos(self):  # bogus 55 missing 92
        have = {}
        nums = list(self.final_pos.keys())
        for x, y in self.pos_ct.items():
            have[x] = self.final_pos[x]
            nums.remove(x)
        self.have_not = {}
        for x in nums:
            self.have_not[x] = self.final_pos[x]
        have = sort_dct_key(have)
        return

    def separable(self):
        if self.line == 757:
            bb = 8

        tword = re.sub(r'[<>\s]', '', self.tword)
        end = len(self.infos) if len(self.infos) < self.line + 30 else self.line + 30
        begin = 0 if self.line < 30 else self.line - 30

        for e, t in en(self.infos[begin:end]):
            if len(t) < 2 or t == self.info:
                pass
            else:
                str1 = re.sub(r'[<>\s0-9]', '', t[2])
                if str1 == tword:
                    lemma = self.parse_lemma(t[1])
                    if self.split_separable(self.tword, t[2], e, t, lemma):
                        return 1
                    elif self.delay_sum:
                        return 0
                    else:
                        self.add2pos('_sp')
                        return 1
        return 0

    def split_separable(self, tword, c, idx, info, lemma):
        # todo one of the lemmas should be preceded by _
        pos = info[4]
        start = idx - 30
        if start < 0:
            first = tword
            second = c
            kind = 0
        else:
            first = c
            second = tword
            kind = 1

        if first.count('<') > 1 or \
                first.count('>') > 1 or \
                second.count('<') > 1 or \
                second.count('>') > 1:
            return 0

        f = vgf.outside(first, '<', '>')
        s = vgf.outside(second, '<', '>')
        s = re.sub(r'[0-9]', '', s)
        f = re.sub(r'[0-9]', '', f)

        if not f and s:
            f = re.sub(r'[<>0-9]', '', first)
            if f:
                f = f.replace(s, '').strip()

        elif f and not s:
            s = re.sub(r'[<>0-9]', '', second)
            if s:
                s = s.replace(f, '').strip()

        if s in self.sum2pos_dct or \
                f in self.sum2pos_dct:
            self.delay_sum = 1
            return 0

        if not kind:
            self.tword = s
        else:
            self.tword = f

        if not self.tword:
            bb = 8

        if ' ' in self.tword:
            self.add2pos('_ss')

        else:

            self.tword = s
            if tword[0] == '<':
                self.tlemma = '_' + lemma
                altlemma = lemma + '_'
            else:
                self.tlemma = lemma + '_'
                altlemma = '_' + lemma

            idx2 = idx - 30
            self.auth2work[self.auth][self.work][self.line + idx2][1] = altlemma
            self.add2pos(pos)
        return 1

    def sum2pos(self, word, tpos):
        if word.count('<') > 1:
            p(f'{word} has two <')
            return
        elif word.count('(') > 1:
            p(f'{word} has two (')
            return
        else:
            be = self.get_be(word)
            self.sum_word = be
            pos = self.sum2pos_dct.get(be)
            if not pos:
                p(f'failure: {be}')
                return
            if len(pos) > 1:
                if be == 'esto':
                    return pos[0]
                elif 'im' in tpos:
                    return pos[1]
                elif 'sj' in tpos:
                    return pos[1]
                else:
                    return pos[0]
            else:
                return pos[0]

    def sum2pos2(self, word):
        oword = word
        word2 = self.get_be(word)
        if word2 in self.sum2pos_dct:
            self.sum_word = word2
            return 1
        mark = ['(', ')'] if "(" in word else ["<", ">"]
        if word[0] != mark[0]:
            word = word.replace(mark[1], '')
            word = word.replace(' ' + mark[0], mark[1] + ' ')
            word = mark[0] + word
        else:
            word = word.replace(mark[0], '')
            word = word.replace(mark[1] + ' ', " " + mark[0])
            word = word + mark[1]

        try:
            word2 = self.get_be(word)
        except:
            return 0
        if word2 in self.sum2pos_dct:
            self.sum_word = word2
            self.backwards_sum = 1

            # p(f'success {oword}')
            return 1

    def anticipate_sum_elision(self):
        '''
        this is for those verbs or words whose ending
        est is attached onto, the controversial decision
        was made to not bother with the post of 'est'

        '''

        if self.tword.endswith('st') or \
                self.tword.endswith("'s"):
            try:
                nlemma = self.infos[self.line + 1][1]
                nlemma = self.parse_lemma(nlemma)
                nword = self.infos[self.line + 1][2]
                nword = re.sub(r'[0-9]', '', nword)
                if nlemma == '_sum' and nword == self.tword:
                    if self.tword.endswith("st"):
                        self.tword = self.tword[:-2] + '|st'
                        self.sum_word = 'est'
                    else:
                        self.tword = self.tword[:-2] + '|s'
                        self.sum_word = 'es'

                    self.delay_sum = 1
                    self.elsum = 1
                    self.infos[self.line + 1][0] = '*'
                    self.infos[self.line + 1][1] = '_sum2'
                    return 1
            except:
                pass

    def anticipate_sum(self, pos):
        start = vgf.beg_list(self.infos, self.line - 30)
        stop = vgf.end_list(self.infos, self.line + 30)
        tword2 = self.lasla_match(self.tword)
        found = 0
        if self.backwards_sum:
            bb = 8

        for i in range(self.line - 1, start, -1):
            try:
                oword = self.infos[i][2]
                word = self.lasla_match(oword)
                if word == tword2:
                    if self.sum_word == 'iri':
                        self.infos[i][1] = 'eo2'
                    else:
                        self.infos[i][1] = 'sum2'
                    self.infos[i][4] = pos
                    self.infos[i][2] = self.sum_word
                    self.infos[i][0] = "*"
                    found = 1
                    break
            except:
                pass

        if not found:
            for i in range(self.line + 1, stop):
                try:
                    oword = self.infos[i][2]
                    word = self.lasla_match(oword)
                    if word == tword2:
                        if self.sum_word == 'iri':
                            self.infos[i][1] = 'eo2'
                        else:
                            self.infos[i][1] = 'sum2'
                        self.infos[i][4] = pos
                        self.infos[i][2] = self.sum_word
                        self.infos[i][0] = "*"
                        found = 1
                        break
                except:
                    pass

        if not found:
            if "(" in self.word:
                self.errors['paren'].append([self.word])
            else:
                pass
                '''
                if there is no sum counterpart then that is because two participles share a single auxiliary such as
                imploratus adoptione et accitus es
                '''
                # self.errors['sum_counter'].append([self.work, self.line, self.infos[start:stop]])
                # p(f'no sum counterpart for {self.tword}')
        else:
            if not self.backwards_sum:
                pass
                self.tword = re.sub(r'<.*>', '', self.tword).strip()
            else:
                s = vgf.inside(self.tword, '<', '>')
                if not s:
                    self.add2errors('inside', self.tword)
                else:
                    pass
                    self.tword = s

        return

    def sum2pos3(self, tpos):
        pos = self.sum2pos_dct.get(self.sum_word)
        if not pos:
            p(f'failure: {self.sum_word}')
            return
        if len(pos) > 1:
            if self.sum_word == 'esto':
                return pos[0]
            elif 'im' in tpos:
                return pos[1]
            elif 'sj' in tpos:
                return pos[1]
            else:
                return pos[0]
        else:
            return pos[0]

    def get_be(self, word):
        if word.count('<') > 1:
            s = get_between_sline(word)
            return self.get_be2(s, word)

        else:
            mark = ['(', ')'] if "(" in word else ["<", ">"]
            s = word[word.index(mark[0]) + 1:word.index(mark[1])]
            if ' ' in s:
                return self.get_be2(s, word)
            else:
                return s

    def get_be2(self, s, word):
        lst = s.split()
        t = set(x for x in lst if x in self.sum2pos_dct)
        if not len(t) == 1:
            self.add2errors('no_be', word)
        else:
            return list(t)[0]

    def find_missing(self):
        if self.kind == 'set':
            self.all_lems = set(x[0] for x in self.all_lines)
            self.lem2info = defaultdict(list)
            for x in self.new_lines:
                self.lem2info[x[1]].append([x[0], x[2]])

        missing_pdct = defaultdict(list)
        b = 0
        for x in self.have_not.values():
            p(f'{b} of {len(self.have_not)}')
            b += 1
            num = self.final_pos_rv[x]
            for lem in self.all_lems:
                itms = self.lem2forms.get(lem)
                if itms:
                    for itm in itms:
                        lst = itm.get(num)
                        if lst:
                            lst1 = lst[0]
                            lst1 = [norm_str_jv(t) for t in lst1]
                            for word in lst1:
                                obj = self.word2pos.get(word)
                                if obj:
                                    for pair in obj:
                                        missing_pdct[x].append([word] + pair)

        not_attested = set(self.have_not.values()) - set(missing_pdct.keys())
        nota = {}
        for x in not_attested:
            num = self.final_pos_rv[x]
            nota[num] = x
        nota = sort_dct_key(nota)

        return

    def nouns(self, info, ignore=0):
        num2 = info[4][1:]
        try:
            decl = num2[0]
        except:
            decl = '0'



        if not ignore:
            self.add_lem2decl(self.tlemma, ('n', decl))
        try:
            number = 's' if num2[2] == '1' else 'p'
        except:
            number = 'N'
        try:
            case = self.num2case[num2[1]]
        except:
            case = 'A'

        if case == 'i':
            self.add2pos('inv')
        else:
            self.add2pos(number + case)





    def verb_dcts(self):
        self.num2tense = {
            '0': '&',
            '1': 'pr',
            '2': 'ip',
            '3': 'fu',
            '4': 'pf',
            '5': 'pp',
            '6': 'fp',
            '7': 'xp',
            '8': 'xq',
            '9': 'xf',
        }
        self.num2mood = {
            '0': '*',
            '1': 'in',
            '2': 'im',
            '3': 'sj',
            '4': 'pr',
            '5': 'gv',
            '6': 'gd',
            '7': 'if',
            '8': 'su',
            '9': 'sup',
        }
        self.num2gen = {
            '0': ')',
            '1': 'C',
            '2': 'f',
            '3': 'B',
            '4': 'm',
            '5': 'X',
            '6': 'n',
            ' ': ' '
        }

        self.num2gen2 = {
            '0': ' ',
            '1': 'm',
            '2': 'f',
            '3': 'm',
            '4': 'm',
            '5': 'm',
            '6': 'n',
            ' ': 'm'
        }


        self.num2case = {
            '1': 'n',
            '2': 'v',
            '3': 'a',
            '4': 'g',
            '5': 'd',
            '6': 'b',
            '7': 'l',
            '8': 'i',
            ' ': ' '
        }

    def verb_dcts2(self):
        self.num2tense2 = {
            '0': '',
            '1': 'present',
            '2': 'imperfect',
            '3': 'future',
            '4': 'perfect',
            '5': 'past perfect',
            '6': 'future perfect',
            '7': 'periphrastic 1',
            '8': 'periphrastic 2',
            '9': 'periphrastic 3',
        }
        self.num2mood2 = {
            '0': '',
            '1': 'indicative',
            '2': 'imperative',
            '3': 'subjunctive',
            '4': 'participle',
            '5': 'gerundive',
            '6': 'gerund',
            '7': 'infinitive',
            '8': 'supine_n',
            '9': 'supine_m',
        }

        self.num2case2 = {
            '1': 'nominative',
            '2': 'vocative',
            '3': 'accusative',
            '4': 'genitive',
            '5': 'dative',
            '6': 'ablative',
            '7': 'locative',
            '8': 'indeclinable',
            ' ': ' '
        }

        self.num2gen2a = {
            '0': '',
            '1': 'common',
            '2': 'feminine',
            '3': 'masc_fem',
            '4': 'masculine',
            '5': 'masc_neuter',
            '6': 'neuter',
            ' ': 'unknown'
        }

    def add_lem2decl(self, k, v):

        if type(v) == int:
            bb = 8
        elif v[1] == ' ':
            v = (v[0], '0')

        if k not in self.lemma2decl:
            self.lemma2decl[k] = defaultdict(int)
        self.lemma2decl[k][v] += 1

    def sum_conditions(self):
        if self.sum_paren:
            return 1
        if reg(r'[<\(]', self.tword) and self.sum2pos2(self.tword):
            return 1
        if self.tlemma in ['_sum2', '_eo2']:
            return 1
        if self.anticipate_sum_elision():
            return 1

    def verbs(self, info):
        num2 = info[4]
        self.sum_word = ''
        gender = 'G'
        para = 0
        person = 'P'
        case = 'A'
        mood = 'M'
        voice = 'V'
        tense = 'T'
        number = 'N'
        for e, x in en(num2[1:]):
            self.verbs_forms[e].add(x)
            if e == 0:
                decl = x
            elif e == 1:
                if not hl(x):
                    pass
                else:
                    case = self.num2case[x]
            elif e == 2:
                number = 's' if x == '1' else 'p'

            elif e == 3:
                pass
            elif e == 4:
                mood = self.num2mood[x]

            elif e == 5:
                if x in ['7', '8', '9']:
                    para = 1
                tense = self.num2tense[x]
            elif e == 6:
                if x in ['1', '3', '4']:
                    voice = 'ac'
                elif x == '2':
                    voice = 'pa'
            elif e == 7:
                if not hl(x):
                    person = '0'
                else:
                    person = int(x)
                    if number == 'p':
                        person += 3

                    person = str(person)
            elif e == 11:
                gender = self.num2gen[x]

        self.add_lem2decl(self.tlemma, ('v', decl))

        if self.sum_conditions():
            total = ''
            if self.sum_word == 'esse':
                total = 'if.pr.ac'
            elif self.sum_word == 'fuisse':
                total = 'if.pf.ac'
            if self.sum_word == 'iri':
                total = 'if.fu.pa'
                self.anticipate_sum(total)
                self.add2pos('sup')

            elif mood == 'if' or self.sum_word in ['esse', 'fuisse']:

                if tense == 'xf':
                    total2 = f'fu.{voice}.{number}{gender}{case}'
                elif tense == 'xq':
                    total2 = f'pf.pa.{number}{gender}{case}'
                else:
                    total2 = f'{tense}.{voice}.{number}{gender}{case}'
                if not self.elsum and not self.sum_paren:
                    self.anticipate_sum(total)
                self.add2pos(total2)
            else:
                tpos = f'{person}.{tense}.{mood}.{voice}'
                tpos2 = f'{person}.{mood}.{voice}'
                if not self.sum_paren:
                    if not total:
                        total = self.sum2pos3(tpos)
                    totala = f'{total[0]}{total[4:]}'
                    if tpos2 != totala:
                        self.sum_errors[tpos].append([tpos, self.sum_word, total])

                if tense in ['xq', 'xp']:
                    total2 = f'pf.pa.{number}{gender}{case}'
                else:
                    total2 = f'pf.pa.{number}{gender}{case}'
                if not self.elsum and not self.sum_paren:
                    self.anticipate_sum(total)
                self.add2pos(total2)

        elif mood == 'su':
            self.add2pos('su')
        elif mood == 'sup':
            self.add2pos('sup')


        elif para:
            if tense == 'xf':
                tense = 'fu'
            total = f'{person}.{tense}.{mood}.{voice}'
            self.add2pos(total)

        elif mood == 'gd':
            self.add2pos('gd.' + case)

        elif mood == 'if':
            self.add2pos(f'{mood}.{tense}.{voice}')

        elif mood == 'gv':
            total = f'gv.{number}{gender}{case}'
            self.add2pos(total)

        elif mood not in ['pr', 'gv']:
            total = f'{person}.{tense}.{mood}.{voice}'
            self.add2pos(total)

        else:

            if mood == 'pr' and tense == 'fu':
                mood = 'fu'
            elif mood == 'pr' and voice == 'pa':
                mood = 'pf'
            if not case:
                num3 = num2[:8] + '   ' + num2[11]
                self.no_case[num3].append(info)

            total = f'{mood}.{voice}.{number}{gender}{case}'
            self.add2pos(total)

    def uverbs(self, num2, dct):
        for e, x in en(num2[1:]):
            dct[e].add(x)

    def adj(self, info, ignore=0):
        num2 = info[4]
        gender = 'G'
        degree = 'D'
        number = 'N'
        decl = 'E'
        case = 'A'
        for e, x in en(num2[1:]):
            self.adj_forms[e].add(x)
            if e == 0:
                decl = x
            elif e == 1:
                case = self.num2case[x]
            elif e == 2:
                number = 'p' if x == '2' else 's'
            elif e == 3:
                if x == '1':
                    degree = 'p'
                elif x == '2':
                    degree = 'c'
                elif x == '3':
                    degree = 's'
            elif e == 11:
                gender = self.num2gen[x]

                if x == '0':
                    self.add2errors('zero', self.tword)

        if gender == 'C':
            bb=8

        if case == 'i':
            self.add2pos('inv')

        elif degree in ['c', 's']:
            total = number + gender + case + degree


            self.add2pos(total)
        else:
            total = number + gender + case
            if self.tlemma == 'nemo':
                total = total.replace(' ','')

            self.add2pos(total)
        if not ignore:
            self.add_lem2decl(self.tlemma, ('a', decl))

    def adv(self, info):
        if not self.kind == 'set':
            num2 = info[4]
        else:
            num2 = info[2]

        try:
            num = num2[1]
        except:
            num = '0'

        self.add_lem2decl(self.tlemma, ('d', num))

        dct = {
            '1': 'd',
            '2': 'ac',
            '3': 'as'
        }
        self.add2pos(dct[num2[4]])

    def other(self, info):
        if not self.kind == 'set':
            num2 = info[4]
        else:
            num2 = info[2]

        try:
            num = num2[1]
        except:
            num = '0'

        self.add_lem2decl(self.tlemma, (num2[0], num))

        if num2[0] in ['E', 'G']:
            self.nouns(info, 1)

        elif num2[0] in ['F', 'I', 'J', 'L', 'K', 'H',
                         ]:
            self.adj(info, 1)

        elif num2[0] in ['O', 'R', 'N', 'E', 'U', 'P', 'Q', 'T', 'S']:
            self.owords[num2[0]].add(self.tword)
            self.add2pos('d')

        else:
            self.add2pos('m')

    def review(self):
        self.auth2work = pi.open_pickle(f'{fold}lasla_db2', 1)
        self.auth2work_db = pi.open_pickle(f'{fold}lasla_db', 1)
        self.errors = pi.open_pickle(f'{fold}lasla_errors', 1)
        self.anomalies = pi.open_pickle(f'{fold}lasla_anomalies', 1)
        self.bogus_pos = pi.open_pickle(f'{fold}bogus_pos', 1)
        self.final_pos = pi.open_pickle(f'{fold}final_pos', 1)
        self.final_pos_rv = {v: k for k, v in self.final_pos.items()}

    def temp23(self):
        s = 'BellumGallicum_CaesBG1'
        lst = []
        for e, x in en(self.auth2work['Caesar'][s]):
            if x[0] != '!':
                lst.append(x[2])
            if e > 100:
                break
        to.from_lst2txt(lst, f'{fold}temp_ceasar')
        vgf.open_txt_file(f'{fold}temp_ceasar')

    def review_errors2(self):
        b = 0
        for self.auth, works in self.auth2work.items():
            for self.work, self.infos in works.items():
                b += 1
                p(b)
                for self.line, info in en(self.infos):
                    word = info[2]
                    lem = info[1]
                    pos = info[4]
                    cat = info[0]
                    if cat != '!' and pos not in ['m']:
                        if not lem:
                            self.add2errors('no_lem', word)
                        elif lem in ['#', '0']:
                            pass
                        elif lem[0] == '_' or lem[-1] == '_':
                            pass
                        elif reg(r'[^a-z]', lem[:-1]):
                            self.add2errors('blem', info[2])
                        elif len(info) != 5:
                            self.add2errors('blen', info[2])
                        elif "$" in word:
                            self.add2errors('bad_greek', info[2])
                        elif pos not in self.final_pos_rv \
                                and not '_' in pos:
                            self.add2errors('bpos', pos)
        return

    def normal_words(self):
        b = 0
        for self.auth, works in self.auth2work.items():
            for self.work, self.infos in works.items():
                b += 1
                p(b)
                for self.line, info in en(self.infos):
                    info[2] = re.sub(r'<.*>', '', info[2]).strip()
                    if '|st' in info[2]:
                        info[2] = info[2][:-3]
                    elif '|s' in info[2]:
                        info[2] = info[2][:-2]

                    if info[1] in ['_sum2', '_eo2']:
                        info[1] = info[1][1:]

    def review_errors(self):
        '''


        '''

        lst1 = []
        kind = 0

        if self.kind not in ['small','debug']:
            if kind:
                self.auth2work_db = pi.open_pickle(f'{fold}lasla_db2', 1)
            else:
                self.auth2work_db = self.auth2work

        errors2 = defaultdict(list)
        errors3 = defaultdict(list)
        words = {}
        for x, l in self.errors.items():

            for e, y in en(l):
                # if e > 100:
                #     break

                infos = self.auth2work_db[y[1]][y[2]]
                infos2 = self.auth2work[y[1]][y[2]]
                lst = vgf.cut_list(infos, y[3], -10)
                lst4 = vgf.cut_list(infos2, y[3], -10)
                yu = copy.deepcopy(y)
                y.append(lst)
                yu.append(lst4)
                errors2[x].append(y)
                errors3[x].append(yu)
                words[y[0]] = lst4

        # lst2 = []
        # errors3 = defaultdict(list)
        #
        # for x, l in self.errors.items():
        #     for e, y in en(l):
        #         infos = self.auth2work[y[1]][y[2]]
        #         lst = vgf.cut_list(infos, y[3], -10)
        #         lst2.append([(y[0], lst)])
        return


class fix_ill_formed_words:
    def __init__(self):
        pass

    def begin_fiw(self):
        pass

    def research_bad_words(self):
        bad_words = []
        bad_words = []
        paren = []
        spaces = set()
        abb = {}
        aux = set()
        bad_period = []
        for x, y in self.lafreq.items():
            word = x[1]
            lemma = x[0]
            if reg(r'[\(\<]', word):
                try:
                    if self.sum2pos2(word):
                        aux.add(word)
                        pass
                    else:
                        paren.append(x)
                except:
                    paren.append(x)

            elif ' ' in word:
                spaces.add(x)

            elif ('9' in lemma or '8' in lemma) and word.count('.') == 1 and \
                    word[-1] == '.':
                pass
            elif word.count('.') == 1 and \
                    word[-1] == '.':
                abb[x] = y
            elif '.' in word:
                bad_period.append(word)
            elif reg(r'[^A-Za-z]', word):
                bad_words.append(x)

        lst = [aux, paren, spaces, abb, bad_period, bad_words]
        pi.save_pickle(lst, f'{fold}ill_formed_words', 1)
        pi.save_pickle(self.bogus_pos, f'{fold}bogus_pos', 1)

        return


def fake_enclitics_lasla():
    lem2forms_pos = pi.open_pickle(f'{fold}lem2forms_pos_ab', 1)
    enclit = to.from_txt2lst(f'{fold}reference/enclitics')
    enclit.remove('st')
    enclit.remove('ue')
    fake_enclitics = defaultdict(dict)
    b = 0
    for x, y in lem2forms_pos.items():
        b += 1
        vgf.print_intervals(b, 100, len(lem2forms_pos))
        for pos, dct in y.items():
            for lem, dct2 in dct.items():
                dct2 = dct2[0]
                for num, lsts in dct2.items():
                    pos = lsts[1]
                    words = lsts[0]
                    for word in words:
                        wordu = norm_str_jv(word)
                        if reg(r'[aeioum]st$', wordu):
                            fake_enclitics['st'].setdefault(wordu, []).append((x, pos))
                        else:
                            for en in enclit:
                                if word.endswith(en):
                                    fake_enclitics[en].setdefault(wordu, []).append((x, pos))

    for en, words in fake_enclitics.items():
        for word, lst in words.items():
            dct = defaultdict(list)
            for x in lst:
                dct[x[0]].append(x[1])
            fake_enclitics[en][word] = dct

    pi.save_pickle(fake_enclitics, f'{fold}fake_enclitics_lasla', 1)


if eval(not_execute_on_import):
    if vgf.pycharm():
        args = [0, 'c', '', 'IX', '', 0, 0]
    else:
        args = vgf.get_arguments()

    if args[1] == 'c':
        ins = convert2txt()
        ins.kind = ''
        ins.begin_ct()
    elif args[1] == 'r':
        ins = convert2txt()
        ins.begin3()
    else:
        p ('wrong argument')



from copy import deepcopy
import unicodedata
from bglobals import *



def normalize_unicode(lines):
    """ Transform lines into diacritics bytes free unicode
    """
    return unicodedata.normalize('NFKD', lines).encode('ASCII', 'ignore').decode()


#############################################################
# Convert morphos.la
# Each line of Morphos.la represents a declension name
#############################################################
search = re.compile(r'[^-]')
morphos = {}
test = 0

with open(f"{fold}morphos.txt") as f:
    for line in f.readlines():
        line = line.strip()
        if line.startswith("!") or len(line) < 2 or ":" not in line:
            pass
        else:
            # Person
            # If we have a person, we know it's a verb
            line = line \
                .replace("1ère", "v1-------") \
                .replace("2ème", "v2-------") \
                .replace("3ème", "v3-------")

            # Number
            line = line \
                .replace("singulier", "--s------") \
                .replace("pluriel", "--p------")

            # Tense
            line = line \
                .replace("présent", "---p-----") \
                .replace("imparfait", "---i-----") \
                .replace("parfait", "---r-----") \
                .replace("PQP", "---l-----") \
                .replace("futur antérieur", "---t-----") \
                .replace("futur", "---f-----")

            # Mood
            line = line \
                .replace("indicatif", "----i----") \
                .replace("subjonctif", "----s----") \
                .replace("infinitif", "v---n----") \
                .replace("impératif", "----m----") \
                .replace("gérondif", "----g----") \
                .replace("adjectif verbal", "----g----") \
                .replace("participe", "g---p----") \
                .replace("supin en -um", "----u----") \
                .replace("supin en -u", "----u----")

            # Voice
            line = line \
                .replace("actif", "-----a---") \
                .replace("passif", "-----p---")

            # Gender
            line = line \
                .replace("masculin", "------m--") \
                .replace("féminin", "------f--") \
                .replace("neutre", "------n--")

            # Case
            line = line \
                .replace("nominatif", "-------n-") \
                .replace("génitif", "-------g-") \
                .replace("accusatif", "-------a-") \
                .replace("datif", "-------d-") \
                .replace("ablatif", "-------b-") \
                .replace("vocatif", "-------v-") \
                .replace("locatif", "-------l-")

            # Degree
            line = line \
                .replace("comparatif", "a-------c") \
                .replace("superlatif", "a-------s") \
                .replace("positif", "a-------p")

            line = line.replace("461:", "416:--------")

            # Then we merge the tags
            new_tag = "---------"
            index, tags = line.split(":")
            tags = tags.split()

            for tag in tags:
                for x in search.finditer(tag):
                    i = x.start()
                    new_tag = new_tag[:i] + tag[i] + new_tag[i + 1:]

            morphos[index] = new_tag



#############################################################
# Convert modeles.la
# Line starting with $ are variable that can be reused
# Set of line starting with model are models
# R:int:int,int (Root number, Character to remove to get canonical form, number of character to add to get the root)
#  -> eg. : for uita, R:1:1,0 would get root 1, 1 character to remove, 0 to add -> uit
#  -> eg. : for epulae, R:1:2,0 would get root 1, 2 character to remove, 0 to add : epul
#############################################################


def parse_range(des_number):
    """ Range
    :return: Int reprenting element of the range
    """
    ids = []
    for des_group in des_number.split(","):  # When we have ";", we should parse it normally
        if "-" in des_group:
            start, end = tuple([int(x) for x in des_group.split("-")])
            ids += list(range(start, end + 1))
        else:
            ids += [int(des_group)]
    return ids

def get_model(pos):
    model = {
        "R": {},
        "abs": [],  # Unused desinence if inherits
        "des": {},  # Dict of desinences
        "suf": [],  # Dict of Suffixes
        "sufd": [],  # Possible endings
        "pos": pos  # Possible endings
    }
    return model


def convert_models(lines, tst=0):

    old2new = {}

    models = {}

    __R = re.compile("^R:(?P<root>\d+):(?P<remove>-|\w+)[,:]?(?P<add>\w+)?", flags=re.UNICODE)
    if not tst:
        __des = re.compile("^des[\+]?:(?P<range>[\d\-,]+):(?P<root>\d+):(?P<des>[\w"+cobr+"[\-,;]+)?$")
    else:
        __des = re.compile("^des[\+]?:(?P<range>[\d\-,]+):(?P<root>\d+):(?P<des>[\w\-,;]+)?$", flags=re.UNICODE)
    last_model = None
    variable_replacement = {}
    current_pos = 'n'
    lpos = 'n'
    real_last_model = ''
    for lineno, line in enumerate(lines):
        line = line.strip()
        if "des:189-200:2:$lupus" in line:
            bb=8

        if line.startswith('!;'):
            old2new[line[2:]] = last_model
        if line.startswith('pos:'):
            current_pos = line[4]
            models[last_model]['pos'] = current_pos

        # If we get a variable
        if line.startswith("$"):
            # We split the line on =
            var, rep = tuple(line.split("="))
            # We create a replacement variable
            variable_replacement[var] = rep
        elif len(line) > 0 and not line.startswith("!"):
            if line.startswith("modele:"):

                last_model = line[7:]
                models[last_model] = get_model(current_pos)
            elif line.startswith("pere:"):
                # Inherits from parent
                models[last_model].update(
                    deepcopy(models[line[5:]])
                )


            elif line.startswith("R:"):
                # Still do not know how to deal with "K"
                root, remove, chars = __R.match(line).groups()

                if remove == "-":
                    # ToDo: Check how radical with "-" should work
                    continue
                if chars == "0":
                    chars = ""
                models[last_model]["R"][root] = [remove, chars]
            elif line.startswith("des"):
                # We have new endings
                # des:range:root_number:list_of_des
                # First we apply desinence variables replacement
                if "$" in line:
                    for var, rep in variable_replacement.items():
                        # First we replace +$
                        if tst:
                            line = re.sub(
                                r"(\w+)(\+?\{})".format(var),
                                lambda x: (
                                    ";".join([x.group(1) + r for r in rep.split(";")])
                                ),
                                line, flags=re.UNICODE
                            )
                        else:
                            line = re.sub(
                                r"(\w+)(\+?\{})".format(var),
                                lambda x: (
                                    ";".join([x.group(1) + r for r in rep.split(";")])
                                ),
                                line
                            )

                        line = line.replace(var, rep)
                        if "$" not in line:
                            break
                try:
                    des_number, root, des = __des.match(line).groups()
                except AttributeError as E:
                    # print(line, lineno)
                    raise E
                if not des:
                    # ToDo : "Deal with empty value in desinence ?"
                    continue
                nums = parse_range(des_number)

                desinence = des.split(";")
                last_des = []
                for desinence_index, desinence_num in enumerate(nums):
                    if desinence_index >= len(desinence):
                        # We might have ranges where number of item < ranges. This seems to mean last item is repeated.
                        current_des = last_des
                    else:
                        current_des = desinence[desinence_index].replace("-", "").split(",")

                    if current_des:
                        desinence_int = int(desinence_num)
                        # If we have des+, we add to the known desinence
                        if line.startswith("des+") and desinence_int in models[last_model]["des"]:
                            models[last_model]["des"][desinence_int].append((root, current_des))
                        else:
                            models[last_model]["des"][desinence_int] = [(root, current_des)]
                        last_des = current_des
                    else:
                        print("Line %s : No desinence for id %s (%s)" % (lineno, desinence_num, last_model))

            elif line.startswith("abs:"):
                models[last_model]["abs"] = parse_range(line[4:])  # Add the one we should not find as desi
            elif line.startswith("suf:"):
                rng, suf = tuple(line[4:].split(":"))
                models[last_model]["suf"].append([suf, list(parse_range(rng))])  # Suffixes are alternative ending
            elif line.startswith("sufd:"):
                models[last_model]["sufd"] += line[5:].split(",")  # Sufd are suffix always present
            else:
                if line.startswith("pos"):
                    continue
                # print(line.split(":")[0], lineno)

    pi.save_pickle(old2new,f'{fold}old2new_mods',1)
    return models


def test_diff(new, old, new_lemmas=[]):
    dec_old = CollatinusDecliner(old)
    dec_new = CollatinusDecliner(new)
    num2pos = dec_old._data['pos']
    pos2num = {v:k for k,v in num2pos.items()}

    for k, v in old.items():
        if k == "sequor":
            bb = 8

        new_obj = new[k]
        for x, y in new_obj['des'].items():
            old_des = v['des'][x]
            if old_des != y:
                pass
                # p(k, x, old_des, y)
        if k in dec_old.co_lemmas0:
            dold = dec_old.decline(k)
            dnew = dec_new.decline(k)
            if dnew != dold:
                dctn = {v:k for k,v in dnew}
                dcto = {v:k for k,v in dold}

                # p(k)
                for x, n in dctn.items():
                    o = dcto.get(x)
                    if o and n != o:
                        # p (x,pos2num[x], n,o)
                        bb = 8



    return


def get_norm_mod(file, tst=0):
    with open(file) as f:
        if tst:
            lines = normalize_unicode(f.read()).split("\n")
        else:
            lines = f.read().split("\n")
        lines += to.from_txt2lst(f'{fold}modeles5')
        norm_models = convert_models(lines,tst)
    return norm_models


def test_mod(norm_models):
    assert norm_models["fortis"]["des"][13] == [("4", [''])], \
        "Root 4, Empty string (originally '-') expected, found %s %s" % norm_models["fortis"]["des"][13][0]
    assert norm_models["fortis"]["des"][51] == [("1", ["iorem"])], \
        "Root 4, iorem expected, found %s %s" % norm_models["fortis"]["des"][50][0]
    assert norm_models["dico"]["des"][181] == [("0", ["e"]), ("0", [""])], \
        "[(0, e), (0, ''), found %s %s " % tuple([str(x) for x in norm_models["dico"]["des"][181]])
    assert norm_models["edo"]["des"][122] == [("0", ["is"]), ("3", ["es"])], \
        "[(0, is), (3, es), found %s %s " % tuple([str(x) for x in norm_models["edo"]["des"][122]])

def test_old_mod():

    file = f"{fold}modeles4.txt"
    file_old = f"{fold}modeles.txt"
    norm_models_old = get_norm_mod(file_old, 1)
    norm_models = get_norm_mod(file, 1)
    test_diff(norm_models, norm_models_old)





############################################
#
#   Get the lemma converter
#
# lemma=lemma|model|genitive/infectum|perfectu|morpho indications
#
############################################

def parseLemma(lines):
    """
    :param lines:
    :param normalize:
    :return:
    # ToDo: Fix issue with
    Caeres2=Cāeres|miles|Cāerĭt,Cāerĭtēt||ĭtis, (-ētis), f.|2
    Caerēs=Cāerēs|diues|Cāerĭt||ĭtis|2
    """

    lemmas = {}
    lemma_without_variations = re.compile(
        r"^(?P<lemma>\w+\d?){1}(?:\=(?P<quantity>[\w,]+))?\|"
        r"(?P<model>\w+)?\|"
        r"[-]*(?P<geninf>[\w,]+)?[-]*\|"
        r"[-]*(?P<perf>[\w,]+)?[-]*\|"
        r"(?P<lexicon>.*)?",
        flags=re.UNICODE
    )
    for lineno, line in enumerate(lines):
        if not line.startswith("!") and "|" in line:
            if line.count("|") != 4:
                # We need to clean up the mess
                # Some line lacks a |
                # I assume this means we need;ĭbŭs to add as many before the dictionary
                should_have = 4
                missing = should_have - line.count("|")
                last_one = line.rfind("|")
                line = line[:last_one] + "|" * missing + line[last_one:]

            result = lemma_without_variations.match(line)
            if result:
                result = result.groupdict(default=None)
                # we always normalize the key
                lemmas[normalize_unicode(result["lemma"])] = result
            else:
                print("Unable to parse lemma", line)
    return lemmas
#
# def get_lemmas():
#     with open(f"{fold}lemmes.txt") as f:
#         lines = normalize_unicode(f.read()).split("\n")
#         lemmas = parseLemma(lines)
#
#     assert lemmas["volumen"]["geninf"] == "volumin"
#     assert lemmas["volumen"]["lemma"] == "volumen"
#     assert lemmas["volumen"]["model"] == "corpus"
#
#     with open(f"{fold}lem_ext.txt") as f:
#         lines = normalize_unicode(f.read()).split("\n")
#         lemmas.update(parseLemma(lines))


def begin_mod():
    file = f"{fold}modeles4.txt"
    norm_models = get_norm_mod(file)
    pi.save_pickle(norm_models, f'{fold}mmodels', 1)

begin_mod()